@@ -34,6 +34,7 @@
 #include "canvas.h"
 #include "canvasgesture_resize.h"
 #include "contextmenu.h"
+#include "hruler.h"
 #include "customfdialog.h"
 #include "fpoint.h"
 #include "fpointarray.h"
@@ -67,6 +68,8 @@ CanvasMode_Edit::CanvasMode_Edit(ScribusView* view) : CanvasMode(view), m_ScMW(v
 	frameResizeHandle = -1;
 	m_blinker = new QTimer(view);	
 	connect(m_blinker, SIGNAL(timeout()), this, SLOT(blinkTextCursor()));
+	connect(view->horizRuler, SIGNAL(MarkerMoved(double, double)), this, SLOT(rulerPreview(double, double)));
+	m_RulerGuide = -1;
 }
 
 inline bool CanvasMode_Edit::GetItem(PageItem** pi)
@@ -87,6 +90,18 @@ void CanvasMode_Edit::blinkTextCursor()
 	}
 }
 
+void CanvasMode_Edit::rulerPreview(double base, double xp)
+{
+	PageItem* currItem;
+	if (m_doc->appMode == modeEdit && GetItem(&currItem))
+	{
+		QPoint oldP = m_canvas->canvasToLocal(QPointF(m_RulerGuide, currItem->yPos()));
+		m_RulerGuide = base + xp;
+		QPoint p = m_canvas->canvasToLocal(QPointF(m_RulerGuide, currItem->yPos() + currItem->height()));
+		m_canvas->update(QRect(oldP.x()-2, oldP.y(), p.x()+2, p.y()));
+	}
+}
+
 void CanvasMode_Edit::drawControls(QPainter* p)
 {
 	commonDrawControls(p, false);
@@ -95,7 +110,31 @@ void CanvasMode_Edit::drawControls(QPainter* p)
 	{
 		PageItem_TextFrame* textframe = currItem->asTextFrame();
 		if (textframe)
+		{
+			if (m_RulerGuide >= 0)
+			{
+				p->save();
+				p->translate(currItem->xPos(), currItem->yPos());
+				p->rotate(currItem->rotation());
+				p->setPen(QPen(Qt::blue, 1.0 / m_canvas->scale(), Qt::DashLine, Qt::FlatCap, Qt::MiterJoin));
+				p->setClipRect(QRectF(0.0, 0.0, currItem->width(), currItem->height()));
+				p->setBrush(Qt::NoBrush);
+				p->setRenderHint(QPainter::Antialiasing);
+				if (currItem->imageFlippedH())
+				{
+					p->translate(currItem->width(), 0);
+					p->scale(-1, 1);
+				}
+				if (currItem->imageFlippedV())
+				{
+					p->translate(0, currItem->height());
+					p->scale(1, -1);
+				}
+				p->drawLine(m_RulerGuide - currItem->xPos(), 0, m_RulerGuide - currItem->xPos(), currItem->height());
+				p->restore();
+			}
 			drawTextCursor(p, textframe);
+		}
 	}
 }
 
@@ -149,7 +188,8 @@ void CanvasMode_Edit::drawTextCursor ( QPainter *p, PageItem_TextFrame* textfram
 			// Happens often when typing directly into frame.
 			// And the cursor curses nothing, vertigo.
 			textCursorPos = textframe->lastInFrame();
-			if(textframe->itemText.text(textCursorPos) == SpecialChars::PARSEP)
+			QChar textCursorChar = textframe->itemText.text(textCursorPos);
+			if (textCursorChar == SpecialChars::PARSEP || textCursorChar == SpecialChars::LINEBREAK)
 			{
 				// The cursor must be moved to the beginning of the next line
 				FRect bbox = textframe->itemText.boundingBox ( textCursorPos );
@@ -180,7 +220,7 @@ void CanvasMode_Edit::drawTextCursor ( QPainter *p, PageItem_TextFrame* textfram
 				dy  += lineSpacing;
 				dy1 += lineSpacing;
 			}
-			else if((textframe->itemText.text(textCursorPos) == SpecialChars::COLBREAK))
+			else if (textCursorChar == SpecialChars::COLBREAK)
 			{
 				// A bit tricky :)
 				// We want to position the cursor at the beginning of the next column, if any.
@@ -306,6 +346,7 @@ void CanvasMode_Edit::activate(bool fromGesture)
 	{
 		m_view->update();
 	}
+	m_RulerGuide = -1;
 	PageItem * it(0);
 	if (GetItem(&it))
 	{
@@ -319,7 +360,10 @@ void CanvasMode_Edit::deactivate(bool forGesture)
 //	qDebug() << "CanvasMode_Edit::deactivate" << forGesture;
 	m_view->redrawMarker->hide();
 	if (!forGesture)
+	{
+		m_RulerGuide = -1;
 		m_blinker->stop();
+	}
 }
 
 void CanvasMode_Edit::mouseDoubleClickEvent(QMouseEvent *m)
@@ -331,6 +375,7 @@ void CanvasMode_Edit::mouseDoubleClickEvent(QMouseEvent *m)
 	PageItem *currItem = 0;
 	if (GetItem(&currItem) && (m_doc->appMode == modeEdit) && currItem->asTextFrame())
 	{
+		currItem->asTextFrame()->lastUndoAction = PageItem::NOACTION;
 		//CB if annotation, open the annotation dialog
 		if (currItem->isAnnotation())
 		{
@@ -338,9 +383,43 @@ void CanvasMode_Edit::mouseDoubleClickEvent(QMouseEvent *m)
 		}
 		//otherwise, select between the whitespace
 		else
-		{	//Double click in a frame to select a word
-			oldCp = currItem->CPos;
-			currItem->CPos = currItem->itemText.selectWord(currItem->CPos);
+		{
+			if (m->modifiers() & Qt::ControlModifier)
+			{
+				int start=0, stop=0;
+
+				if (m->modifiers() & Qt::ShiftModifier)
+				{//Double click with Ctrl+Shift in a frame to select few paragraphs
+					uint oldPar = currItem->itemText.nrOfParagraph(oldCp);
+					uint newPar = currItem->itemText.nrOfParagraph(currItem->CPos);
+					if (oldPar < newPar)
+					{
+						start = currItem->itemText.startOfParagraph(oldPar);
+						stop = currItem->itemText.endOfParagraph(newPar);
+					}
+					else
+					{
+						start = currItem->itemText.startOfParagraph(newPar);
+						stop = currItem->itemText.endOfParagraph(oldPar);
+					}
+				}
+				else
+				{//Double click with Ctrl in a frame to select a paragraph
+					oldCp = currItem->CPos;
+					uint nrPar = currItem->itemText.nrOfParagraph(oldCp);
+					start = currItem->itemText.startOfParagraph(nrPar);
+					stop = currItem->itemText.endOfParagraph(nrPar);
+				}
+				currItem->itemText.deselectAll();
+				currItem->itemText.extendSelection(start, stop);
+				currItem->CPos = stop;
+			}
+			else
+			{	//Double click in a frame to select a word
+				oldCp = currItem->CPos;
+				currItem->CPos = currItem->itemText.selectWord(currItem->CPos);
+			}
+			currItem->HasSel = (currItem->itemText.lengthOfSelection() > 0);
 		}
 	}
 	else
@@ -519,11 +598,12 @@ void CanvasMode_Edit::mousePressEvent(QMouseEvent *m)
 	if (GetItem(&currItem))
 	{
 //		m_view->slotDoCurs(false);
-		if (!currItem->locked() && !currItem->asLine())
+		if ((!currItem->locked() || currItem->isTextFrame()) && !currItem->asLine())
 		{
 			FPoint canvasPoint = m_canvas->globalToCanvas(m->globalPos());
 			if (m_canvas->frameHitTest(QPointF(canvasPoint.x(), canvasPoint.y()), currItem) < 0)
 			{
+				m_doc->m_Selection->delaySignalsOn();
 				m_view->Deselect(true);
 				bool wantNormal = true;
 				if (SeleItem(m))
@@ -540,13 +620,17 @@ void CanvasMode_Edit::mousePressEvent(QMouseEvent *m)
 						qApp->changeOverrideCursor(QCursor(Qt::ArrowCursor));
 					}
 					if (currItem->asTextFrame())
+					{
 						m_view->slotSetCurs(m->globalPos().x(), m->globalPos().y());
+						currItem->asTextFrame()->lastUndoAction = PageItem::NOACTION;
+					}
 				}
 				else
 				{
 					m_view->requestMode(submodePaintingDone);
 					qApp->changeOverrideCursor(QCursor(Qt::ArrowCursor));
 				}
+				m_doc->m_Selection->delaySignalsOff();
 				if (wantNormal)
 					m_view->requestMode(modeNormal);
 				return;
@@ -556,6 +640,7 @@ void CanvasMode_Edit::mousePressEvent(QMouseEvent *m)
 		//CB Where we set the cursor for a click in text frame
 		if (currItem->asTextFrame())
 		{
+			currItem->asTextFrame()->lastUndoAction = PageItem::NOACTION;
 			inText = m_view->slotSetCurs(m->globalPos().x(), m->globalPos().y());
 			//CB If we clicked outside a text frame to go out of edit mode and deselect the frame
 			if (!inText)
@@ -564,26 +649,57 @@ void CanvasMode_Edit::mousePressEvent(QMouseEvent *m)
 				m_view->Deselect(true);
 				//m_view->slotDoCurs(true);
 				m_view->requestMode(modeNormal);
+				if (currItem->isTextFrame())
+					currItem->asTextFrame()->lastUndoAction = PageItem::NOACTION;
 				return;
 			}
 
 			if (m->button() != Qt::RightButton)
 			{
-				currItem->asTextFrame()->deselectAll();
+				//currItem->asTextFrame()->deselectAll();
 				//<<CB Add in shift select to text frames
-				if (m->modifiers() & Qt::ShiftModifier && currItem->itemText.lengthOfSelection() > 0)
+				if (m->modifiers() & Qt::ShiftModifier)
 				{
-					if (currItem->CPos < (currItem->itemText.startOfSelection() + currItem->itemText.endOfSelection()) / 2)
-						oldP = currItem->itemText.startOfSelection();
-					else 
-						oldP = currItem->itemText.endOfSelection();
-					currItem->asTextFrame()->itemText.extendSelection(oldP, currItem->CPos);
-					oldCp = currItem->CPos;
+					if (currItem->itemText.lengthOfSelection() > 0)
+					{
+						if (currItem->CPos < (currItem->itemText.startOfSelection() + currItem->itemText.endOfSelection()) / 2)
+						{
+							if (m->modifiers() & Qt::ControlModifier)
+								currItem->CPos = currItem->itemText.startOfParagraph(currItem->itemText.nrOfParagraph(currItem->CPos));
+							oldP = currItem->itemText.startOfSelection();
+						}
+						else
+						{
+							if (m->modifiers() & Qt::ControlModifier)
+								currItem->CPos = currItem->itemText.endOfParagraph(currItem->itemText.nrOfParagraph(currItem->CPos));
+							oldP = currItem->itemText.endOfSelection();
+						}
+						currItem->asTextFrame()->itemText.extendSelection(oldP, currItem->CPos);
+						currItem->asTextFrame()->lastUndoAction = PageItem::NOACTION;
+						oldCp = currItem->CPos;
+					}
+					else
+					{
+						int dir=1;
+						if (oldCp > currItem->CPos)
+							dir=-1;
+						if (m->modifiers() & Qt::ControlModifier) //no selection but Ctrl+Shift+click still select paragraphs
+						{
+							if (dir == 1)
+								currItem->CPos = currItem->itemText.endOfParagraph(currItem->itemText.nrOfParagraph(currItem->CPos));
+							else
+								currItem->CPos = currItem->itemText.startOfParagraph(currItem->itemText.nrOfParagraph(currItem->CPos));
+						}
+						currItem->asTextFrame()->ExpandSel(dir, oldP);
+						currItem->asTextFrame()->lastUndoAction = PageItem::NOACTION;
+						oldCp = oldP;
+					}
 				}
 				else //>>CB
 				{
 					oldCp = currItem->CPos;
 					currItem->itemText.deselectAll();
+					currItem->asTextFrame()->lastUndoAction = PageItem::NOACTION;
 					currItem->HasSel = false;
 				}
 				currItem->emitAllToGUI();
@@ -598,7 +714,9 @@ void CanvasMode_Edit::mousePressEvent(QMouseEvent *m)
 					if (!cc.isNull())
 					{
 						// K.I.S.S.:
+						currItem->oldCPos = 0;
 						currItem->itemText.insertChars(0, cc, true);
+						currItem->asTextFrame()->updateUndo(PageItem::INS,cc);
 						if (m_doc->docHyphenator->AutoCheck)
 							m_doc->docHyphenator->slotHyphenate(currItem);
 						m_ScMW->BookMarkTxT(currItem);
@@ -616,6 +734,7 @@ void CanvasMode_Edit::mousePressEvent(QMouseEvent *m)
 		else if (!currItem->asImageFrame() || 
 				 m_canvas->frameHitTest(QPointF(mousePointDoc.x(),mousePointDoc.y()), currItem) < 0)
 		{
+			m_doc->m_Selection->delaySignalsOn();
 			m_view->Deselect(true);
 			if (SeleItem(m))
 			{
