@@ -5,35 +5,36 @@ a copyright and/or license notice that predates the release of Scribus 1.3.2
 for which a new license (GPL+exception) is in place.
 */
 #include "picsearch.h"
-#include "picsearch.moc"
-#include <qpixmap.h>
-#include <qpainter.h>
-#include "scribusstructs.h"
-#include "scimage.h"
+#include <QPixmap>
+#include <QPainter>
+
+#include "cmsettings.h"
 #include "commonstrings.h"
+#include "scimage.h"
 #include "scpaths.h"
+#include "scribusstructs.h"
 #include "util.h"
-
-extern QPixmap loadIcon(QString nam);
+#include "util_color.h"
+#include "util_formats.h"
+#include "util_icon.h"
 
 
-PicSearch::PicSearch(QWidget* parent, const QString & fileName, const QStringList & avalableFiles)
-	: PicSearchBase( parent, "PicSearch", true, 0 ),
-	currentImage(QString())
+PicSearch::PicSearch(QWidget* parent, const QString & fileName, const QStringList & avalableFiles) : QDialog(parent), currentImage(QString())
 {
-	setIcon(loadIcon("AppIcon.png"));
+	setupUi(this);
+	setModal(true);
 	cancelButton->setText(CommonStrings::tr_Cancel);
 	previewLabel->hide();
 	fileNameLabel->setText(fileName);
 
-	for (uint i = 0; i < avalableFiles.count(); ++i)
-		foundFilesBox->insertItem( QDir::convertSeparators(avalableFiles[i]) );
+	for (int i = 0; i < avalableFiles.count(); ++i)
+		foundFilesBox->addItem( QDir::convertSeparators(avalableFiles[i]) );
 
 	// signals and slots connections
 	connect(cancelButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
 	connect(useButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
 	connect(previewCheckBox, SIGNAL( clicked() ), this, SLOT( previewCheckBox_clicked() ) );
-	connect(foundFilesBox, SIGNAL(clicked(QListBoxItem*)), this, SLOT(foundFilesBox_clicked(QListBoxItem*)));
+	connect(foundFilesBox, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(foundFilesBox_clicked(QListWidgetItem*)));
 }
 
 void PicSearch::previewCheckBox_clicked()
@@ -48,11 +49,11 @@ void PicSearch::previewCheckBox_clicked()
 		previewLabel->hide();
 }
 
-void PicSearch::foundFilesBox_clicked(QListBoxItem *c)
+void PicSearch::foundFilesBox_clicked(QListWidgetItem *c)
 {
 	if (c == NULL)
 		return;
-	currentImage = ScPaths::separatorsToSlashes(c->text());
+	currentImage = QDir::fromNativeSeparators(c->text());
 	if (previewCheckBox->isChecked())
 		createPreview();
 	useButton->setEnabled(true);
@@ -65,13 +66,13 @@ void PicSearch::createPreview()
 	int w = 200;
 	int h = 200;
 	bool mode = false;
-	QString ext = fi.extension(false).lower();
+	QString ext = fi.suffix().toLower();
 	if (ext.isEmpty())
 		ext = getImageType(currentImage);
 	ScImage im;
 	//No doc to send data anyway, so no doc to get into scimage.
-	CMSettings cms(0, "", 0);
-	if (im.LoadPicture(currentImage, cms, false, false, ScImage::Thumbnail, 72, &mode))
+	CMSettings cms(0, "", Intent_Perceptual);
+	if (im.LoadPicture(currentImage, 1, cms, false, false, ScImage::Thumbnail, 72, &mode))
 	{
 		int ix,iy;
 		if ((im.imgInfo.exifDataValid) && (!im.imgInfo.exifInfo.thumbnail.isNull()))
@@ -93,8 +94,8 @@ void PicSearch::createPreview()
 		{
 			double sx = im.width() / static_cast<double>(w-5);
 			double sy = im.height() / static_cast<double>(h-44);
-			im2 = sy < sx ?  im.smoothScale(qRound(im.width() / sx), qRound(im.height() / sx)) :
-								im.smoothScale(qRound(im.width() / sy), qRound(im.height() / sy));
+			im2 = sy < sx ? im.scaled(qRound(im.width() / sx), qRound(im.height() / sx), Qt::IgnoreAspectRatio, Qt::SmoothTransformation)
+						: im.scaled(qRound(im.width() / sy), qRound(im.height() / sy), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
 		}
 		else
 			im2 = im.qImage(); // no need to copy
@@ -107,26 +108,10 @@ void PicSearch::createPreview()
 		p.drawText(2, h-29, tr("Size:")+" "+tmp.setNum(ix)+" x "+tmp2.setNum(iy));
 		p.drawText(2, h-17, tr("Resolution:")+" "+tmp.setNum(xres)+" x "+tmp2.setNum(yres)+" "+ tr("DPI"));
 		QString cSpace;
-		if (((ext == "pdf") || (ext == "eps") || (ext == "epsi") || (ext == "ps")) && (im.imgInfo.type != 7))
+		if ((extensionIndicatesPDF(ext) || extensionIndicatesEPSorPS(ext)) && (im.imgInfo.type != ImageType7))
 			cSpace = tr("Unknown");
 		else
-		{
-			switch (im.imgInfo.colorspace)
-			{
-				case 0:
-					cSpace = tr("RGB");
-					break;
-				case 1:
-					cSpace = tr("CMYK");
-					break;
-				case 2:
-					cSpace = tr("Grayscale");
-					break;
-				case 3:
-					cSpace = tr("Duotone");
-					break;
-			}
-		}
+			cSpace=colorSpaceText(im.imgInfo.colorspace);
 		p.drawText(2, h-5, tr("Colorspace:")+" "+cSpace);
 		p.end();
 		repaint();
