@@ -6,26 +6,26 @@ for which a new license (GPL+exception) is in place.
 */
 #include "scriptercore.h"
 
-#include <qglobal.h>
-#include <qwidget.h>
-#include <qstring.h>
-#include <qapplication.h>
-#include <qmessagebox.h>
-#include <qtextcodec.h>
-#include <qdom.h>
-#include <qtextstream.h>
+#include <QGlobalStatic>
+#include <QWidget>
+#include <QString>
+#include <QApplication>
+#include <QMessageBox>
+#include <QTextCodec>
+#include <QByteArray>
+#include <QPixmap>
 #include <cstdlib>
-#include <qtextedit.h>
 
 #include "runscriptdialog.h"
 #include "helpbrowser.h"
-#include "mpalette.h" //TODO Move the calls to this to a signal
-#include "seiten.h" //TODO Move the calls to this to a signal
+#include "propertiespalette.h" //TODO Move the calls to this to a signal
+#include "pagepalette.h" //TODO Move the calls to this to a signal
 #include "layers.h" //TODO Move the calls to this to a signal
-#include "tree.h" //TODO Move the calls to this to a signal
+#include "outlinepalette.h" //TODO Move the calls to this to a signal
 #include "menumanager.h"
 #include "pconsole.h"
 #include "scraction.h"
+#include "scribuscore.h"
 #include "scpaths.h"
 #include "selection.h"
 #include "prefsfile.h"
@@ -33,39 +33,30 @@ for which a new license (GPL+exception) is in place.
 #include "prefstable.h"
 #include "prefsmanager.h"
 
-#include "scriptercore.moc"
-
 ScripterCore::ScripterCore(QWidget* parent)
 {
 	pcon = new PythonConsole(parent);
-	menuMgr = ScMW->scrMenuMgr;
 	scrScripterActions.clear();
 	scrRecentScriptActions.clear();
 	returnString = "init";
 
-	scrScripterActions.insert("scripterExecuteScript", new ScrAction(QObject::tr("&Execute Script..."), QKeySequence(), this, "scripterExecuteScript"));
-	scrScripterActions.insert("scripterShowConsole", new ScrAction(QObject::tr("Show &Console"), QKeySequence(), this, "scripterShowConsole"));
-	scrScripterActions.insert("scripterAboutScript", new ScrAction(QObject::tr("&About Script..."), QKeySequence(), this, "scripterAboutScript"));
+	scrScripterActions.insert("scripterExecuteScript", new ScrAction(QObject::tr("&Execute Script..."), QKeySequence(), this));
+	scrScripterActions.insert("scripterShowConsole", new ScrAction(QObject::tr("Show &Console"), QKeySequence(), this));
+	scrScripterActions.insert("scripterAboutScript", new ScrAction(QObject::tr("&About Script..."), QKeySequence(), this));
+
+	scrScripterActions["scripterExecuteScript"]->setMenuRole(QAction::NoRole);
+	scrScripterActions["scripterShowConsole"]->setMenuRole(QAction::NoRole);
+	scrScripterActions["scripterAboutScript"]->setMenuRole(QAction::NoRole);
 
 	scrScripterActions["scripterShowConsole"]->setToggleAction(true);
+	scrScripterActions["scripterShowConsole"]->setChecked(false);
 
-	QObject::connect( scrScripterActions["scripterExecuteScript"], SIGNAL(activated()) , this, SLOT(runScriptDialog()) );
+	QObject::connect( scrScripterActions["scripterExecuteScript"], SIGNAL(triggered()) , this, SLOT(runScriptDialog()) );
 	QObject::connect( scrScripterActions["scripterShowConsole"], SIGNAL(toggled(bool)) , this, SLOT(slotInteractiveScript(bool)) );
-	QObject::connect( scrScripterActions["scripterAboutScript"], SIGNAL(activated()) , this, SLOT(aboutScript()) );
-
-	menuMgr->createMenu("Scripter", QObject::tr("&Script"));
-	menuMgr->addMenuToMenuBarAfter("Scripter","Extras");
-	menuMgr->createMenu("ScribusScripts", QObject::tr("&Scribus Scripts"), "Scripter");
-	menuMgr->addMenuItem(scrScripterActions["scripterExecuteScript"], "Scripter");
-	menuMgr->createMenu("RecentScripts", QObject::tr("&Recent Scripts"), "Scripter");
-	menuMgr->addMenuSeparator("Scripter");
-	menuMgr->addMenuItem(scrScripterActions["scripterShowConsole"], "Scripter");
-	menuMgr->addMenuItem(scrScripterActions["scripterAboutScript"], "Scripter");
+	QObject::connect( scrScripterActions["scripterAboutScript"], SIGNAL(triggered()) , this, SLOT(aboutScript()) );
 
 	SavedRecentScripts.clear();
 	ReadPlugPrefs();
-	buildScribusScriptsMenu();
-	buildRecentScriptsMenu();
 
 	QObject::connect(pcon, SIGNAL(runCommand()), this, SLOT(slotExecute()));
 	QObject::connect(pcon, SIGNAL(paletteShown(bool)), this, SLOT(slotInteractiveScript(bool)));
@@ -74,6 +65,22 @@ ScripterCore::ScripterCore(QWidget* parent)
 ScripterCore::~ScripterCore()
 {
 	SavePlugPrefs();
+	delete pcon;
+}
+
+void ScripterCore::addToMainWindowMenu(ScribusMainWindow *mw)
+{
+	menuMgr = mw->scrMenuMgr;
+	menuMgr->createMenu("Scripter", QObject::tr("&Script"));
+	menuMgr->addMenuToMenuBarBefore("Scripter","Windows");
+	menuMgr->createMenu("ScribusScripts", QObject::tr("&Scribus Scripts"), "Scripter");
+	menuMgr->addMenuItem(scrScripterActions["scripterExecuteScript"], "Scripter");
+	menuMgr->createMenu("RecentScripts", QObject::tr("&Recent Scripts"), "Scripter");
+	menuMgr->addMenuSeparator("Scripter");
+	menuMgr->addMenuItem(scrScripterActions["scripterShowConsole"], "Scripter");
+	menuMgr->addMenuItem(scrScripterActions["scripterAboutScript"], "Scripter");
+	buildScribusScriptsMenu();
+	buildRecentScriptsMenu();
 }
 
 
@@ -82,15 +89,15 @@ void ScripterCore::buildScribusScriptsMenu()
 	QString pfad = ScPaths::instance().scriptDir();
 	QString pfad2;
 	pfad2 = QDir::convertSeparators(pfad);
-	QDir ds(pfad2, "*.py", QDir::Name, QDir::Files | QDir::NoSymLinks);
+	QDir ds(pfad2, "*.py", QDir::Name | QDir::IgnoreCase, QDir::Files | QDir::NoSymLinks);
 	if ((ds.exists()) && (ds.count() != 0))
 	{
 		for (uint dc = 0; dc < ds.count(); ++dc)
 		{
 			QFileInfo fs(ds[dc]);
-			QString strippedName=fs.baseName(false);
-			scrScripterActions.insert(strippedName, new ScrAction( ScrAction::RecentScript, QIconSet(), strippedName, QKeySequence(), this, strippedName));
-			connect( scrScripterActions[strippedName], SIGNAL(activatedData(QString)), this, SLOT(StdScript(QString)) );
+			QString strippedName=fs.baseName();
+			scrScripterActions.insert(strippedName, new ScrAction( ScrAction::RecentScript, QPixmap(), QPixmap(), strippedName, QKeySequence(), this));
+			connect( scrScripterActions[strippedName], SIGNAL(triggeredData(QString)), this, SLOT(StdScript(QString)) );
 			menuMgr->addMenuItem(scrScripterActions[strippedName], "ScribusScripts");
 		}
 	}
@@ -100,17 +107,17 @@ void ScripterCore::buildScribusScriptsMenu()
 
 void ScripterCore::rebuildRecentScriptsMenu()
 {
-	for( QMap<QString, QGuardedPtr<ScrAction> >::Iterator it = scrRecentScriptActions.begin(); it!=scrRecentScriptActions.end(); ++it )
+	for( QMap<QString, QPointer<ScrAction> >::Iterator it = scrRecentScriptActions.begin(); it!=scrRecentScriptActions.end(); ++it )
 		menuMgr->removeMenuItem((*it), "RecentScripts");
 
 	scrRecentScriptActions.clear();
-	uint max = QMIN(PrefsManager::instance()->appPrefs.RecentDCount, RecentScripts.count());
+	uint max = qMin(PrefsManager::instance()->appPrefs.RecentDCount, RecentScripts.count());
 	for (uint m = 0; m < max; ++m)
 	{
 		QString strippedName=RecentScripts[m];
 		strippedName.remove(QDir::separator());
-		scrRecentScriptActions.insert(strippedName, new ScrAction( ScrAction::RecentScript, QIconSet(), RecentScripts[m], QKeySequence(), this, strippedName));
-		connect( scrRecentScriptActions[strippedName], SIGNAL(activatedData(QString)), this, SLOT(RecentScript(QString)) );
+		scrRecentScriptActions.insert(strippedName, new ScrAction( ScrAction::RecentScript, QPixmap(), QPixmap(), RecentScripts[m], QKeySequence(), this));
+		connect( scrRecentScriptActions[strippedName], SIGNAL(triggeredData(QString)), this, SLOT(RecentScript(QString)) );
 		menuMgr->addMenuItem(scrRecentScriptActions[strippedName], "RecentScripts");
 	}
 }
@@ -121,7 +128,7 @@ void ScripterCore::buildRecentScriptsMenu()
 	scrRecentScriptActions.clear();
 	if (SavedRecentScripts.count() != 0)
 	{
-		uint max = QMIN(PrefsManager::instance()->appPrefs.RecentDCount, SavedRecentScripts.count());
+		uint max = qMin(PrefsManager::instance()->appPrefs.RecentDCount, SavedRecentScripts.count());
 		for (uint m = 0; m < max; ++m)
 		{
 			QFileInfo fd(SavedRecentScripts[m]);
@@ -129,8 +136,8 @@ void ScripterCore::buildRecentScriptsMenu()
 			{
 				QString strippedName=SavedRecentScripts[m];
 				strippedName.remove(QDir::separator());
-				scrRecentScriptActions.insert(strippedName, new ScrAction( ScrAction::RecentScript, QIconSet(), SavedRecentScripts[m], QKeySequence(), this, strippedName));
-				connect( scrRecentScriptActions[strippedName], SIGNAL(activatedData(QString)), this, SLOT(RecentScript(QString)) );
+				scrRecentScriptActions.insert(strippedName, new ScrAction( ScrAction::RecentScript, QPixmap(), QPixmap(), SavedRecentScripts[m], QKeySequence(), this));
+				connect( scrRecentScriptActions[strippedName], SIGNAL(triggeredData(QString)), this, SLOT(RecentScript(QString)) );
 				menuMgr->addMenuItem(scrRecentScriptActions[strippedName], "RecentScripts");
 			}
 		}
@@ -139,23 +146,20 @@ void ScripterCore::buildRecentScriptsMenu()
 
 void ScripterCore::FinishScriptRun()
 {
+	ScribusMainWindow* ScMW=ScCore->primaryMainWindow();
 	if (ScMW->HaveDoc)
 	{
 		ScMW->propertiesPalette->setDoc(ScMW->doc);
-		ScMW->propertiesPalette->updateCList();
-		ScMW->propertiesPalette->Spal->setFormats(ScMW->doc);
-		ScMW->propertiesPalette->SetLineFormats(ScMW->doc);
-		ScMW->propertiesPalette->updateColorList();
-		ScMW->layerPalette->setLayers(&ScMW->doc->Layers, ScMW->doc->activeLayer());
+		ScMW->layerPalette->setDoc(ScMW->doc);
 		ScMW->outlinePalette->setDoc(ScMW->doc);
 		ScMW->outlinePalette->BuildTree();
 		ScMW->pagePalette->setView(ScMW->view);
 		ScMW->pagePalette->Rebuild();
 		ScMW->doc->RePos = true;
-		QPixmap pgPix(10, 10);
+/*		QImage pgPix(10, 10, QImage::Format_ARGB32);
 		QRect rd = QRect(0,0,9,9);
 		ScPainter *painter = new ScPainter(&pgPix, pgPix.width(), pgPix.height());
-		for (uint azz=0; azz<ScMW->doc->Items->count(); ++azz)
+		for (int azz=0; azz<ScMW->doc->Items->count(); ++azz)
 		{
 			PageItem *ite = ScMW->doc->Items->at(azz);
 			if (ite->Groups.count() != 0)
@@ -163,7 +167,7 @@ void ScripterCore::FinishScriptRun()
 			else
 				ite->OwnPage = ScMW->doc->OnPage(ite);
 			ite->setRedrawBounding();
-			if ((ite->itemType() == PageItem::TextFrame) || (ite->itemType() == PageItem::PathText) && (!ite->Redrawn))
+			if ((ite->itemType() == PageItem::TextFrame) || (ite->itemType() == PageItem::PathText)) // && (!ite->Redrawn))
 			{
 				if (ite->itemType() == PageItem::PathText)
 				{
@@ -173,16 +177,11 @@ void ScripterCore::FinishScriptRun()
 				}
 				else
 				{
-					if ((ite->BackBox != 0) || (ite->NextBox != 0))
+					if ((ite->prevInChain() != 0) || (ite->nextInChain() != 0))
 					{
 						PageItem *nextItem = ite;
-						while (nextItem != 0)
-						{
-							if (nextItem->BackBox != 0)
-								nextItem = nextItem->BackBox;
-							else
-								break;
-						}
+						while (nextItem->prevInChain() != 0)
+							nextItem = nextItem->prevInChain();
 						ite = nextItem;
 						ite->DrawObj(painter, rd);
 					}
@@ -191,7 +190,7 @@ void ScripterCore::FinishScriptRun()
 				}
 			}
 		}
-		delete painter;
+		delete painter; */
 		ScMW->doc->RePos = false;
 		if (ScMW->doc->m_Selection->count() != 0)
 		{
@@ -210,18 +209,18 @@ void ScripterCore::FinishScriptRun()
 void ScripterCore::runScriptDialog()
 {
 	QString fileName;
-	QString curDirPath = QDir::currentDirPath();
-	RunScriptDialog dia( ScMW, m_enableExtPython );
+	QString curDirPath = QDir::currentPath();
+	RunScriptDialog dia( ScCore->primaryMainWindow(), m_enableExtPython );
 	if (dia.exec())
 	{
 		fileName = dia.selectedFile();
 		slotRunScriptFile(fileName, dia.extensionRequested());
 
-		if (RecentScripts.findIndex(fileName) == -1)
+		if (RecentScripts.indexOf(fileName) == -1)
 			RecentScripts.prepend(fileName);
 		else
 		{
-			RecentScripts.remove(fileName);
+			RecentScripts.removeAll(fileName);
 			RecentScripts.prepend(fileName);
 		}
 		rebuildRecentScriptsMenu();
@@ -248,7 +247,7 @@ void ScripterCore::RecentScript(QString fn)
 	QFileInfo fd(fn);
 	if (!fd.exists())
 	{
-		RecentScripts.remove(fn);
+		RecentScripts.removeAll(fn);
 		rebuildRecentScriptsMenu();
 		return;
 	}
@@ -258,22 +257,25 @@ void ScripterCore::RecentScript(QString fn)
 
 void ScripterCore::slotRunScriptFile(QString fileName, bool inMainInterpreter)
 {
-	PyThreadState *stateo, *state;
+	PyThreadState *stateo = NULL;
+	PyThreadState *state = NULL;
 	QFileInfo fi(fileName);
-	QCString na = fi.fileName().latin1();
+	QByteArray na = fi.fileName().toLocal8Bit();
 	// Set up a sub-interpreter if needed:
 	if (!inMainInterpreter)
 	{
-		ScMW->ScriptRunning = true;
-		qApp->setOverrideCursor(QCursor(waitCursor), false);
+		ScCore->primaryMainWindow()->propertiesPalette->unsetDoc();
+		ScCore->primaryMainWindow()->pagePalette->setView(NULL);
+		ScCore->primaryMainWindow()->setScriptRunning(true);
+		qApp->changeOverrideCursor(QCursor(Qt::WaitCursor));
 		// Create the sub-interpreter
 		// FIXME: This calls abort() in a Python debug build. We're doing something wrong.
 		stateo = PyEval_SaveThread();
 		state = Py_NewInterpreter();
 		// Chdir to the dir the script is in
-		QDir::setCurrent(fi.dirPath(true));
+		QDir::setCurrent(fi.absolutePath());
 		// Init the scripter module in the sub-interpreter
-		initscribus(ScMW);
+		initscribus(ScCore->primaryMainWindow());
 	}
 	// Make sure sys.argv[0] is the path to the script
 	char* comm[2];
@@ -291,22 +293,25 @@ void ScripterCore::slotRunScriptFile(QString fileName, bool inMainInterpreter)
 		qDebug("Failed to get __main__ - aborting script");
 	else
 	{
+		// Path separators need to be escaped on Windows
+		QString escapedAbsPath  = QDir::toNativeSeparators(fi.absolutePath()).replace("\\", "\\\\");
+		QString escapedFileName = QDir::toNativeSeparators(fileName).replace("\\", "\\\\");
 		// FIXME: If filename contains chars outside 7bit ascii, might be problems
 		PyObject* globals = PyModule_GetDict(m);
 		// Build the Python code to run the script
-		QString cm = QString("from __future__ import division\n");
-		cm        += QString("import sys\n");
+		//QString cm = QString("from __future__ import division\n"); removed due #5252 PV
+		QString cm = QString("import sys\n");
 		cm        += QString("import cStringIO\n");
 		/* Implementation of the help() in pydoc.py reads some OS variables
 		 * for output settings. I use ugly hack to stop freezing calling help()
 		 * in script. pv. */
 		cm        += QString("import os\nos.environ['PAGER'] = '/bin/false'\n"); // HACK
-		cm        += QString("sys.path[0] = \"%1\"\n").arg(fi.dirPath(true));
+		cm        += QString("sys.path[0] = \"%1\"\n").arg(escapedAbsPath);
 		// Replace sys.stdin with a dummy StringIO that always returns
 		// "" for read
 		cm        += QString("sys.stdin = cStringIO.StringIO()\n");
 		cm        += QString("try:\n");
-		cm        += QString("    execfile(\"%1\")\n").arg(fileName);
+		cm        += QString("    execfile(\"%1\")\n").arg(escapedFileName);
 		cm        += QString("except SystemExit:\n");
 		cm        += QString("    pass\n");
 		// Capture the text of any other exception that's raised by the interpreter
@@ -322,7 +327,7 @@ void ScripterCore::slotRunScriptFile(QString fileName, bool inMainInterpreter)
 		// the fact that an exception has ocurred.
 		cm        += QString("    raise\n");
 		// FIXME: if cmd contains chars outside 7bit ascii, might be problems
-		QCString cmd = cm.latin1();
+		QByteArray cmd = cm.toUtf8();
 		// Now run the script in the interpreter's global scope. It'll run in a
 		// sub-interpreter if we created and switched to one earlier, otherwise
 		// it'll run in the main interpreter.
@@ -348,8 +353,8 @@ void ScripterCore::slotRunScriptFile(QString fileName, bool inMainInterpreter)
 				// Display a dialog to the user with the exception
 				QClipboard *cp = QApplication::clipboard();
 				cp->setText(errorMsg);
-				ScMW->closeSplash();
-				QMessageBox::warning(ScMW,
+				ScCore->closeSplash();
+				QMessageBox::warning(ScCore->primaryMainWindow(),
 									tr("Script error"),
 									"<qt><p>"
 									+ tr("If you are running an official script report it at <a href=\"http://bugs.scribus.net\">bugs.scribus.net</a> please.")
@@ -365,36 +370,45 @@ void ScripterCore::slotRunScriptFile(QString fileName, bool inMainInterpreter)
 	{
 		Py_EndInterpreter(state);
 		PyEval_RestoreThread(stateo);
-		qApp->restoreOverrideCursor();
+//		qApp->restoreOverrideCursor();
+		ScCore->primaryMainWindow()->setScriptRunning(false);
 	}
-	ScMW->ScriptRunning = false;
 }
 
 void ScripterCore::slotRunScript(const QString Script)
 {
-	ScMW->ScriptRunning = true;
+	ScCore->primaryMainWindow()->propertiesPalette->unsetDoc();
+	ScCore->primaryMainWindow()->pagePalette->setView(NULL);
+	ScCore->primaryMainWindow()->setScriptRunning(true);
 	inValue = Script;
 	QString cm;
 	cm = "# -*- coding: utf8 -*- \n";
 	if (PyThreadState_Get() != NULL)
 	{
-		initscribus(ScMW);
+		initscribus(ScCore->primaryMainWindow());
 		/* HACK: following loop handles all input line by line.
 		It *should* use I.C. because of docstrings etc. I.I. cannot
 		handle docstrings right.
 		Calling all code in one command:
 		ia = code.InteractiveInterpreter() ia.runsource(getval())
 		works fine in plain Python. Not here. WTF? */
-		cm += ("import cStringIO\n"
-				"scribus._bu = cStringIO.StringIO()\n"
-				"sys.stdout = scribus._bu\n"
-				"sys.stderr = scribus._bu\n"
-				"sys.argv = ['scribus', 'ext']\n" // this is the PySys_SetArgv replacement
-				"for i in scribus.getval().splitlines():\n"
-				"    scribus._ia.push(i)\n"
-				"scribus.retval(scribus._bu.getvalue())\n"
-				"sys.stdout = sys.__stdout__\n"
-				"sys.stderr = sys.__stderr__\n");
+		cm += (
+				"try:\n"
+				"    import cStringIO\n"
+				"    scribus._bu = cStringIO.StringIO()\n"
+				"    sys.stdout = scribus._bu\n"
+				"    sys.stderr = scribus._bu\n"
+				"    sys.argv = ['scribus', 'ext']\n" // this is the PySys_SetArgv replacement
+				"    for i in scribus.getval().splitlines():\n"
+				"        scribus._ia.push(i)\n"
+				"    scribus.retval(scribus._bu.getvalue())\n"
+				"    sys.stdout = sys.__stdout__\n"
+				"    sys.stderr = sys.__stderr__\n"
+				"except SystemExit:\n"
+				"    print 'Catched SystemExit - it is not good for Scribus'\n"
+				"except KeyboardInterrupt:\n"
+				"    print 'Catched KeyboardInterrupt - it is not good for Scribus'\n"
+			  );
 	}
 	// Set up sys.argv
 	/* PV - WARNING: THIS IS EVIL! This code summons a crash - see
@@ -415,24 +429,28 @@ void ScripterCore::slotRunScript(const QString Script)
 	else
 	{
 		PyObject* globals = PyModule_GetDict(m);
-		PyObject* result = PyRun_String(cm.utf8().data(), Py_file_input, globals, globals);
+		PyObject* result = PyRun_String(cm.toUtf8().data(), Py_file_input, globals, globals);
 		if (result == NULL)
 		{
 			PyErr_Print();
-			QMessageBox::warning(ScMW, tr("Script error"),
+			QMessageBox::warning(ScCore->primaryMainWindow(), tr("Script error"),
 					"<qt>" + tr("There was an internal error while trying the "
 					   "command you entered. Details were printed to "
 					   "stderr. ") + "</qt>");
 		}
+		else
+		// Because 'result' may be NULL, not a PyObject*, we must call PyXDECREF not Py_DECREF
+			Py_XDECREF(result);
 	}
-	ScMW->ScriptRunning = false;
+	ScCore->primaryMainWindow()->setScriptRunning(false);
 }
 
 void ScripterCore::slotInteractiveScript(bool visible)
 {
 	QObject::disconnect( scrScripterActions["scripterShowConsole"], SIGNAL(toggled(bool)) , this, SLOT(slotInteractiveScript(bool)) );
 
-	scrScripterActions["scripterShowConsole"]->setOn(visible);
+	scrScripterActions["scripterShowConsole"]->setChecked(visible);
+	pcon->setFonts();
 	pcon->setShown(visible);
 
 	QObject::connect( scrScripterActions["scripterShowConsole"], SIGNAL(toggled(bool)) , this, SLOT(slotInteractiveScript(bool)) );
@@ -484,7 +502,7 @@ void ScripterCore::SavePlugPrefs()
 		qDebug("scriptplugin: Unable to get recent scripts");
 		return;
 	}
-	for (uint i = 0; i < RecentScripts.count(); i++)
+	for (int i = 0; i < RecentScripts.count(); i++)
 		prefRecentScripts->set(i, 0, RecentScripts[i]);
 	// then save more general preferences
 	prefs->set("extensionscripts", m_enableExtPython);
@@ -494,16 +512,16 @@ void ScripterCore::SavePlugPrefs()
 
 void ScripterCore::aboutScript()
 {
-	QString fname = ScMW->CFileDialog(".", tr("Examine Script"), tr("Python Scripts (*.py);;All Files (*)"), "", 0, 0, 0, 0);
+	QString fname = ScCore->primaryMainWindow()->CFileDialog(".", tr("Examine Script"), tr("Python Scripts (*.py *.PY);;All Files (*)"), "", fdNone);
 	if (fname == QString::null)
 		return;
 	QString html("<html><body>");
 	QFileInfo fi = QFileInfo(fname);
 	QFile input(fname);
-	if(!input.open(IO_ReadOnly))
+	if(!input.open(QIODevice::ReadOnly))
 		return;
 	QTextStream intputstream(&input);
-	QString content = intputstream.read();
+	QString content = intputstream.readAll();
 	QString docstring = content.section("\"\"\"", 1, 1);
 	if (!docstring.isEmpty())
 	{
@@ -529,7 +547,7 @@ void ScripterCore::initExtensionScripts()
 
 void ScripterCore::runStartupScript()
 {
-	if ((m_enableExtPython) && (m_startupScript))
+	if ((m_enableExtPython) && (!m_startupScript.isNull()))
 	{
 		if (QFile::exists(this->m_startupScript))
 		{
@@ -538,19 +556,19 @@ void ScripterCore::runStartupScript()
 			this->slotRunScriptFile(this->m_startupScript, true);
 		}
 		else
-			qDebug("Startup script enabled, but couln't find script %s.", m_startupScript.ascii());
+			qDebug("Startup script enabled, but couln't find script %s.", m_startupScript.toAscii().constData());
 	}
 }
 
 void ScripterCore::languageChange()
 {
-	scrScripterActions["scripterExecuteScript"]->setMenuText(QObject::tr("&Execute Script..."));
-	scrScripterActions["scripterShowConsole"]->setMenuText(QObject::tr("Show &Console"));
-	scrScripterActions["scripterAboutScript"]->setMenuText(QObject::tr("&About Script..."));
+	scrScripterActions["scripterExecuteScript"]->setText(QObject::tr("&Execute Script..."));
+	scrScripterActions["scripterShowConsole"]->setText(QObject::tr("Show &Console"));
+	scrScripterActions["scripterAboutScript"]->setText(QObject::tr("&About Script..."));
 
-	menuMgr->setMenuText("Scripter", QObject::tr("&Script"));
-	menuMgr->setMenuText("ScribusScripts", QObject::tr("&Scribus Scripts"));
-	menuMgr->setMenuText("RecentScripts", QObject::tr("&Recent Scripts"));
+	menuMgr->setText("Scripter", QObject::tr("&Script"));
+	menuMgr->setText("ScribusScripts", QObject::tr("&Scribus Scripts"));
+	menuMgr->setText("RecentScripts", QObject::tr("&Recent Scripts"));
 }
 
 bool ScripterCore::setupMainInterpreter()
@@ -559,19 +577,19 @@ bool ScripterCore::setupMainInterpreter()
 		"# -*- coding: utf-8 -*-\n"
 		"import scribus\n"
 		"import sys\n"
+		"import code\n"
 		"sys.path[0] = \"%1\"\n"
 		"import cStringIO\n"
 		"sys.stdin = cStringIO.StringIO()\n"
-		"import code\n"
 		"scribus._ia = code.InteractiveConsole(globals())\n"
 		).arg(ScPaths::instance().scriptDir());
 	if (m_importAllNames)
 		cm += "from scribus import *\n";
-	QCString cmd = cm.utf8();
+	QByteArray cmd = cm.toUtf8();
 	if (PyRun_SimpleString(cmd.data()))
 	{
 		PyErr_Print();
-		QMessageBox::warning(ScMW, tr("Script error"),
+		QMessageBox::warning(ScCore->primaryMainWindow(), tr("Script error"),
 				tr("Setting up the Python plugin failed. "
 				   "Error details were printed to stderr. "));
 		return false;
