@@ -13,36 +13,29 @@ for which a new license (GPL+exception) is in place.
  *
  */
 
-#include <qlistbox.h>
 #include "sccombobox.h"
 
-#ifdef QT_MAC
+#ifdef Q_WS_MAC
 
-#include <qmacstyle_mac.h>
+#include <QMacStyle>
 
 /// use listbox instead of popupmenu for combo boxes
 class ScMacStyle : public QMacStyle {
-	virtual int ScMacStyle::styleHint(StyleHint sh, const QWidget * w, 
-	                                  const QStyleOption & so, 
+	virtual int styleHint(StyleHint sh, 
+	                                  const QStyleOption * so, 
+					  const QWidget * w, 
 	                                  QStyleHintReturn * r) const
 	{
-        	return sh != QStyle::SH_ComboBox_Popup && QMacStyle::styleHint(sh, w, so, r); 
+        	return sh != QStyle::SH_ComboBox_Popup && QMacStyle::styleHint(sh, so, w, r); 
 	}
 };
 #endif
 
-ScComboBox::ScComboBox( QWidget* parent, const char* name ) : QComboBox(false, parent, name) 
+ScComboBox::ScComboBox( QWidget* parent ) : QComboBox(parent)
 {
-#ifdef QT_MAC
+#ifdef Q_WS_MAC
 	setStyle( new ScMacStyle );
 #endif
-	setListBox( new QListBox( this, "in-combo", WType_Popup ) );
+//Qt4	setListBox( new Q3ListBox( this, "in-combo", Qt::WType_Popup ) );
 }
 
-ScComboBox::ScComboBox( bool rw, QWidget* parent, const char* name ) :QComboBox(rw, parent, name)
-{
-#ifdef QT_MAC
-	setStyle( new ScMacStyle );
-#endif
-	setListBox( new QListBox( this, "in-combo", WType_Popup ) );
-}
