@@ -15,29 +15,31 @@ for which a new license (GPL+exception) is in place.
 	version, at your discretion.
 */
 
-#include <qglobal.h>
-#include <qapplication.h>
-#include <qfile.h>
-#include <qfileinfo.h>
-#include <qstrlist.h>
-#include <qstring.h>
-#include <qtextstream.h>
-#include <qfont.h>
-#include <qdict.h>
-#include <qmap.h>
-#include <qdir.h>
-#include <qregexp.h>
+#include <QApplication>
+#include <QDir>
+#include <QDomDocument>
+#include <QFile>
+#include <QFileInfo>
+#include <QFont>
+#include <QGlobalStatic>
+#include <QMap>
+#include <QRegExp>
+#include <QString>
+
 
 #include <cstdlib>
 #include "scfonts.h"
-#include "scfonts_ttf.h"
-#include "scfontmetrics.h"
+#include "fonts/ftface.h"
+#include "fonts/scface_ps.h"
+#include "fonts/scface_ttf.h"
+#include "fonts/scfontmetrics.h"
 
 #include "prefsmanager.h"
 #include "prefsfile.h"
 #include "prefscontext.h"
 #include "prefstable.h"
 #include "scribus.h"
+#include "scribuscore.h"
 #ifdef Q_WS_X11
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -47,443 +49,20 @@ for which a new license (GPL+exception) is in place.
 #include <fontconfig/fontconfig.h>
 #endif
 
-#include "scpaths.h"
-#include "util.h"
-
 #include FT_TRUETYPE_TAGS_H
 #include FT_TRUETYPE_TABLES_H
 
 
-
-/// copied from Freetype's FT_Stream_ReadAt()
-FT_Error ftIOFunc( FT_Stream stream, unsigned long pos, unsigned char* buffer, unsigned long count) 
-{
-	FT_Error  error = FT_Err_Ok;
-	FT_ULong  read_bytes;
- 
-	if ( pos >= stream->size )
-	{
-                qDebug( "ftIOFunc: invalid i/o; pos = 0x%lx, size = 0x%lx\n",
-                                   pos, stream->size );
-               
-		return FT_Err_Invalid_Stream_Operation;
-	}
- 
-	if ( stream->read )
-		read_bytes = stream->read( stream, pos, buffer, count );
-	else
-	{
-		read_bytes = stream->size - pos;
-		if ( read_bytes > count )
-			read_bytes = count;
-
-		memcpy( buffer, stream->base + pos, read_bytes );
-	}
- 
-	stream->pos = pos + read_bytes;
- 
-	if ( read_bytes < count )
-	{
-		qDebug( "ftIOFunc: invalid read; expected %lu bytes, got %lu\n",
-		        count, read_bytes );
-                
-		error = FT_Err_Invalid_Stream_Operation;
-	}
- 
-	return error;
-}
-
-
-
-
-Foi::Foi(QString fam, QString sty, QString alt, QString psname, QString path, int face, bool embedps)
-{
-	isOTF = false;
-	Subset = false;
-	typeCode = Foi::UNKNOWN_TYPE;
-	formatCode = Foi::UNKNOWN_FORMAT;
-	SCName = alt;
-	Family = fam;
-	Effect = sty;
-	cached_RealName = psname;
-	fontFile = path;
-	faceIndex_ = face;
-	EmbedPS = embedps;
-	Alternative = "";
-}
-
-
-FT_Face ftFace() {
-	// dummy for now
-	return NULL;
-}
-
-QString Foi::RealName()
-{
-	return cached_RealName;
-}
-
-bool Foi::EmbedFont(QString &str)
-{
-	str+="Base";
-	return(false);
-}
-
-bool Foi::ReadMetrics()
-{
-	HasMetrics=false;
-	for(int i=0;i<256;++i)
-		CharWidth[i]=0;
-	return(false);
-}
-
-bool Foi::GlNames(QMap<uint, QString> *GList)
-{
-	return GlyNames(this, GList);
-}
-
-void Foi::RawData(QByteArray & bb)
-{
-	FT_Library library;
-	FT_Face    face;
-	bool       error;
-	
-	error = FT_Init_FreeType( &library );
-	if (error)
-		sDebug(QObject::tr("Freetype2 library not available"));
-	else
-		error = FT_New_Face( library, fontFile, faceIndex_, &face );
-	if (error)
-		sDebug(QObject::tr("Font %1 is broken, no embedding").arg(fontFile));
-	else 
-	{
-		FT_Stream fts = face->stream;
-		bb.resize(fts->size);
-		error = ftIOFunc(fts, 0L, reinterpret_cast<FT_Byte *>(bb.data()), fts->size);
-		if (error) 
-		{
-			sDebug(QObject::tr("Font %1 is broken (read stream), no embedding").arg(fontFile));
-			bb.resize(0);
-		}
-		/*
-		if (showFontInformation)
-		{
-			QFile f(fontFile);
-			qDebug(QObject::tr("RawData for Font %1(%2): size=%3 filesize=%4").arg(fontFile).arg(faceIndex_).arg(bb.size()).arg(f.size()));
-		}
-		*/
-	}
-	FT_Done_Face(face);
-	FT_Done_FreeType(library);
-}
-
-
-/*
-void Foi::FontBez()
-{
-	FT_Face face;
-	FT_Library library;
-	QString ts;
-	QPixmap pm(200, 100);
-	bool error;
-	int  pen_x;
-	int YPos = qApp->font().pointSize();
-	uint n;
-	error = FT_Init_FreeType( &library );
-	error = FT_New_Face( library, fontFile, 0, &face );
-	ts = QString(face->family_name) + " " + QString(face->style_name);
-	pm.fill();
-	pen_x = 0;
-	for (n = 0; n < ts.length(); n++)
-		{
-		uint dv = ts[n].unicode();
-		error = FT_Set_Char_Size(	face, 0, qApp->font().pointSize()*64, 72, 72 );
-		FT_Load_Char(face, dv, FT_LOAD_RENDER | FT_LOAD_NO_BITMAP | FT_LOAD_MONOCHROME);
-		QByteArray bd(face->glyph->bitmap.rows * face->glyph->bitmap.pitch);
-		uint yy = 0;
-		uint adv;
-		if ((face->glyph->bitmap.width % 8) == 0)
-			adv = face->glyph->bitmap.width / 8;
-		else
-			adv = face->glyph->bitmap.width / 8 + 1;
-		for (int y = 0; y < face->glyph->bitmap.rows; ++y)
-			{
-			memcpy(bd.data()+yy, face->glyph->bitmap.buffer+(y * face->glyph->bitmap.pitch), adv);
-			yy += adv;
-			}
-		QBitmap bb(face->glyph->bitmap.width, face->glyph->bitmap.rows, (uchar*)bd.data(), false);
-		QPixmap pixm(face->glyph->bitmap.width, face->glyph->bitmap.rows);
-		if (!pixm.isNull())
-			{
-			pixm.fill(Qt::black);
-			pixm.setMask(bb);
-			QPainter p;
-			p.begin(&pm);
-			p.drawPixmap(pen_x+face->glyph->bitmap_left, YPos-face->glyph->bitmap_top, pixm);
-			p.end();
-			}
-		pen_x += face->glyph->advance.x >> 6;
-		}
-	int high = qApp->fontMetrics().height();
-	QPixmap pm2(pen_x, high);
-	bitBlt(&pm2, 0, 0, &pm, 0, 0, pen_x, high);
-	Appearance = pm2;
-	FT_Done_FreeType( library );
-	return;
-}    */
-
-/*
-	Class Foi_postscript
-	Subclass of Foi, for PostScript fonts that could possibly have a .afm file
-	associated with them for metrics information.
-*/
-
-class Foi_postscript : public Foi
-{
-	public:
-		Foi_postscript(QString fam, QString sty, QString alt, QString psname, QString path, int face, bool embedps) :
-		Foi(fam,sty,alt,psname,path,face,embedps), metricsread(false)
-		{
-			HasMetrics=false;
-			HasKern = false;
-			Ascent = "0";
-			CapHeight = "0";
-			Descender = "0";
-			ItalicAngle = "0";
-			StdVW = "1";
-			FontBBox = "0 0 0 0";
-			IsFixedPitch = false;
-			typeCode = TYPE1;
-			HasMetrics=true;
-		}
-
-		virtual QString RealName()
-		{
-			return(Foi::RealName());
-		}
-
-		virtual bool ReadMetrics()  // routine by Franz Schmid - modified by Alastair M. Robinson
-		{
-			if(metricsread)
-				return(HasMetrics);
-			CharWidth.clear();
-			GlyphArray.clear();
-			QString tmp, tmp2, tmp3, tmp4;
-			double x, y;
-			FPointArray outlines;
-			struct GlyphR GRec;
-			bool			error;
-			FT_Library library;
-			FT_ULong  charcode;
-			FT_UInt   gindex;
-			FT_Face   face;
-			isStroked = false;
-			error = FT_Init_FreeType( &library );
-			if (error)
-			{
-				UseFont = false;
-				sDebug(QObject::tr("Font %1 is broken (FreeType2), discarding it").arg(fontPath()));
-				return false;
-			}
-			error = FT_New_Face( library, fontFilePath(), faceIndex(), &face );
-			if (error)
-			{
-				UseFont = false;
-				sDebug(QObject::tr("Font %1 is broken (no Face), discarding it").arg(fontPath()));
-				FT_Done_FreeType( library );
-				return false;
-			}
-			uniEM = static_cast<double>(face->units_per_EM);
-			QString afnm = fontFilePath().left(fontFilePath().length()-3);
-			QFile afm(afnm+"afm");
-			if(!(afm.exists()))
-			{
-				afm.setName(afnm+"Afm");
-				if(!(afm.exists()))
-					afm.setName(afnm+"AFM");
-			}
-			if (afm.exists())
-				error = FT_Attach_File(face, afm.name());
-			HasKern = FT_HAS_KERNING(face);
-			Ascent = tmp.setNum(face->ascender);
-			Descender = tmp.setNum(face->descender);
-			numDescender = face->descender / uniEM;
-			numAscent = face->ascender / uniEM;
-			underline_pos = face->underline_position / uniEM;
-			strikeout_pos = numAscent / 3;
-			strokeWidth = face->underline_thickness / uniEM;
-			CapHeight = Ascent;
-			ItalicAngle = "0";
-			StdVW = "1";
-			FontBBox = tmp.setNum(face->bbox.xMin)+" "+tmp2.setNum(face->bbox.yMin)+" "+tmp3.setNum(face->bbox.xMax)+" "+tmp4.setNum(face->bbox.yMax);
-			IsFixedPitch = face->face_flags & 4;
-			setBestEncoding(face);
-			gindex = 0;
-			charcode = FT_Get_First_Char( face, &gindex );
-			int goodGlyph = 0;
-			int invalidGlyph = 0;
-			while ( gindex != 0 )
-			{
-				error = FT_Load_Glyph( face, gindex, FT_LOAD_NO_SCALE | FT_LOAD_NO_BITMAP );
-				if (error)
-				{
-					++invalidGlyph;
-					sDebug(QObject::tr("Font %1 has broken glyph %2 (charcode %3)").arg(fontPath()).arg(gindex).arg(charcode));
-					charcode = FT_Get_Next_Char( face, charcode, &gindex );
-					continue;
-				}
-				++goodGlyph;
-				double ww = face->glyph->metrics.horiAdvance / uniEM;
-				if (face->glyph->format == FT_GLYPH_FORMAT_PLOTTER)
-					isStroked = true;
-				error = false;
-				outlines = traceChar(face, charcode, 10, &x, &y, &error);
-				if (!error)
-				{
-					CharWidth.insert(charcode, ww);
-					GRec.Outlines = outlines.copy();
-					GRec.x = x;
-					GRec.y = y;
-					GlyphArray.insert(charcode, GRec);
-				}
-				charcode = FT_Get_Next_Char( face, charcode, &gindex );
-			}
-			FT_Done_Face( face );
-			FT_Done_FreeType( library );
-			if (invalidGlyph > 0) {
-				UseFont = false;
-				sDebug(QObject::tr("Font %1 is broken and will be discarded").arg(fontPath()));
-			}
-			HasMetrics=UseFont;
-			metricsread=UseFont;
-			return(HasMetrics);
-		}
-	private:
-		bool metricsread;
-};
-
-/*
-	Class Foi_pfb
-	Subclass of Foi, specifically for Adobe type 1 .pfb fonts.
-	Implements: RealName() and EmbedFont().
-*/
-
-class Foi_pfb : public Foi_postscript
-{
-	public:
-		Foi_pfb(QString fam, QString sty, QString alt, QString psname, QString path, int face, bool embedps) :
-		Foi_postscript(fam,sty,alt,psname,path,face,embedps)
-		{
-			formatCode = PFB;
-		}
-
-		virtual bool EmbedFont(QString &str)
-		{
-			QByteArray bb;
-			RawData(bb);
-			QString tmp2 = "";
-			if ((bb.size() > 2) &&  (bb[0] == char(0x80)) && (static_cast<int>(bb[1]) == 1))
-			{
-				QString tmp3="";
-				QString tmp4 = "";
-				uint posi,cxxc=0;
-				for (posi = 6; posi < bb.size(); ++posi)
-				{
-					if ((bb[posi] == char(0x80)) && (posi+1 < bb.size()) && (static_cast<int>(bb[posi+1]) == 2))
-						break;
-					str += bb[posi];
-				}
-				uint ulen;
-				if (posi+6 < bb.size()) 
-				{
-					ulen = bb[posi+2] & 0xff;
-					ulen |= (bb[posi+3] << 8) & 0xff00;
-					ulen |= (bb[posi+4] << 16) & 0xff0000;
-					ulen |= (bb[posi+5] << 24) & 0xff000000;
-					posi += 6;
-					if (posi + ulen > bb.size())
-						ulen = bb.size() - posi - 1;
-					char linebuf[80];
-					cxxc=0;
-					for (uint j = 0; j < ulen; ++j)
-					{
-						unsigned char u=bb[posi];
-						linebuf[cxxc]=((u >> 4) & 15) + '0';
-						if(u>0x9f) linebuf[cxxc]+='a'-':';
-						++cxxc;
-						u&=15; linebuf[cxxc]=u + '0';
-						if(u>0x9) linebuf[cxxc]+='a'-':';
-						++posi;
-						++cxxc;
-						if (cxxc > 72)
-						{
-							linebuf[cxxc++]='\n';
-							linebuf[cxxc++]=0;
-							str += linebuf;
-							cxxc = 0;
-						}
-					}
-					linebuf[cxxc]=0;
-					str += linebuf;
-					str += "\n";
-				}
-				posi += 6;
-				for (uint j = posi; j < bb.size(); ++j)
-				{
-					if ((bb[j] == static_cast<char>(0x80)) && (j+1 < bb.size()) && (static_cast<int>(bb[j+1]) == 3))
-						break;
-					if(bb[j]=='\r')
-						str+="\n";
-					else
-						str += bb[j];
-				}
-				str += "\n";
-				cxxc = 0;
-				return true;
-			}
-			else 
-			{
-				sDebug(QObject::tr("Font %1 cannot be read, no embedding").arg(fontPath()));
-				return false;
-			}
-		}
-};
-
-/*
-	Class Foi_pfa
-	Subclass of Foi, specifically for Adobe type 1 and type 3 .pfa fonts.
-	Implements: RealName() and EmbedFont().
-*/
-
-class Foi_pfa : public Foi_postscript
-{
-	public:
-		Foi_pfa(QString fam, QString sty, QString alt, QString psname, QString path, int face, bool embedps) :
-		Foi_postscript(fam,sty,alt,psname,path,face,embedps)
-		{
-			formatCode = PFA;
-		}
-		virtual bool EmbedFont(QString &str)
-		{
-			QByteArray bb;
-			RawData(bb);
-			if (bb.size() > 2 && bb[0] == '%' && bb[1] == '!') 
-			{
-				// this is ok since bb will not contain '\0'
-				str.append(bb);
-				return true; 
-			}
-			return false;
-		}
-};
+#include "scpaths.h"
+#include "util.h"
 
 
 
 /***************************************************************************/
 
-SCFonts::SCFonts() : QDict<Foi>(), FontPath(true)
+SCFonts::SCFonts() : QMap<QString,ScFace>(), FontPath()
 {
-	setAutoDelete(true);
+//	insert("", ScFace::none()); // Wtf why inserting an empty entry here ????
 	showFontInformation=false;
 	checkedFonts.clear();
 }
@@ -496,23 +75,22 @@ void SCFonts::updateFontMap()
 {
 	fontMap.clear();
 	SCFontsIterator it( *this );
-	for ( ; it.current(); ++it)
+	for ( ; it.hasNext(); it.next())
 	{
-		if (it.current()->UseFont)
+		if (it.current().usable())
 		{
-			if (fontMap.contains(it.current()->family()))
+			if (fontMap.contains(it.current().family()))
 			{
-				QStringList effect;
-				if (!fontMap[it.current()->family()].contains(it.current()->style()))
+				if (!fontMap[it.current().family()].contains(it.current().style()))
 				{
-					fontMap[it.current()->family()].append(it.current()->style());
+					fontMap[it.current().family()].append(it.current().style());
 				}
 			}
 			else
 			{
-				QStringList effect;
-				effect.append(it.current()->style());
-				fontMap.insert(it.current()->family(), effect);
+				QStringList styles;
+				styles.append(it.current().style());
+				fontMap.insert(it.current().family(), styles);
 			}
 		}
 	}
@@ -527,7 +105,7 @@ void SCFonts::AddPath(QString p)
 {
 	if(p.right(1) != "/")
 	  p += "/";
-	if(FontPath.find(p)==-1)
+	if(!FontPath.contains(p))
 		FontPath.insert(FontPath.count(),p);
 }
 
@@ -538,13 +116,13 @@ void SCFonts::AddScalableFonts(const QString &path, QString DocName)
 	if (path.isEmpty())
 		return;
 	FT_Library library = NULL;
-	QString pathfile;
+	QString pathfile, fullpath;
 	bool error;
 	error = FT_Init_FreeType( &library );
 	QString pathname(path);
 	if ( !pathname.endsWith("/") )
 		pathname += "/";
-	pathname=QDir::convertSeparators(pathname);
+	pathname=QDir::toNativeSeparators(pathname);
 	QDir d(pathname, "*", QDir::Name, QDir::Dirs | QDir::Files | QDir::Readable);
 	if ((d.exists()) && (d.count() != 0))
 	{
@@ -554,43 +132,59 @@ void SCFonts::AddScalableFonts(const QString &path, QString DocName)
 			// over. Skip 'em.
 			if (d[dc] == "." || d[dc] == "..")
 				continue;
-			QFileInfo fi(pathname+d[dc]);
+			fullpath = pathname+d[dc];
+			QFileInfo fi(fullpath);
 			if (!fi.exists())      // Sanity check for broken Symlinks
 				continue;
 			
 			qApp->processEvents();
 			
-			if (fi.isSymLink())
+			bool symlink = fi.isSymLink();
+			if (symlink)
 			{
 				QFileInfo fi3(fi.readLink());
 				if (fi3.isRelative())
 					pathfile = pathname+fi.readLink();
 				else
-					pathfile = fi3.absFilePath();
+					pathfile = fi3.absoluteFilePath();
 			}
 			else
-				pathfile = pathname+d[dc];
+				pathfile = fullpath;
 			QFileInfo fi2(pathfile);
 			if (fi2.isDir()) 
 			{
+				if (symlink)
+				{
+					// Check if symlink points to a parent directory
+					// in order to avoid infinite recursion
+					QString fullpath2 = fullpath, pathfile2 = pathfile;
+					if (ScCore->isWinGUI())
+					{
+						// Ensure both path use same separators on Windows
+						fullpath2 = QDir::toNativeSeparators(fullpath2.toLower());
+						pathfile2 = QDir::toNativeSeparators(pathfile2.toLower());
+					}
+					if (fullpath2.startsWith(pathfile2))
+						continue;
+				}
 				if (DocName.isEmpty())
 					AddScalableFonts(pathfile);
 				continue;
 			}
-			QString ext = fi.extension(false).lower();
-			QString ext2 = fi2.extension(false).lower();
+			QString ext = fi.suffix().toLower();
+			QString ext2 = fi2.suffix().toLower();
 			if ((ext != ext2) && (ext.isEmpty())) 
 				ext = ext2;
 			if ((ext == "ttc") || (ext == "dfont") || (ext == "pfa") || (ext == "pfb") || (ext == "ttf") || (ext == "otf"))
 			{
 				error = AddScalableFont(pathfile, library, DocName);
 			}
-#ifdef FT_MACINTOSH
+#ifdef Q_OS_MAC
 			else if (ext.isEmpty() && DocName.isEmpty())
 			{
 				error = AddScalableFont(pathfile, library, DocName);
 				if (error)
-					error = AddScalableFont(pathfile + "/rsrc",library, DocName);
+					error = AddScalableFont(pathfile + "/..namedfork/rsrc",library, DocName);
 			}
 #endif				
 		}
@@ -598,10 +192,25 @@ void SCFonts::AddScalableFonts(const QString &path, QString DocName)
 	FT_Done_FreeType( library );
 }
 
+
+/*****
+   What to do with font files:
+   - note mod. date
+   - in FontCache?  => load faces from cache
+   - load via FT    => or broken
+   - for all faces:
+       load face
+       get fontinfo (type, names, styles, global metrics)
+       check encoding(s)
+       (calc. hash sum)
+       create cache entry
+ *****/
+
+
 /**
  * tests magic words to determine the fontformat and preliminary fonttype
  */
-void getFontFormat(FT_Face face, Foi::FontFormat & fmt, Foi::FontType & type)
+void getFontFormat(FT_Face face, ScFace::FontFormat & fmt, ScFace::FontType & type)
 {
 	static const char* T42_HEAD      = "%!PS-TrueTypeFont";
 	static const char* T1_HEAD      = "%!FontType1";
@@ -614,54 +223,54 @@ void getFontFormat(FT_Face face, Foi::FontFormat & fmt, Foi::FontType & type)
 	const FT_Stream fts = face->stream;
 	char buf[128];
 	
-	fmt = Foi::UNKNOWN_FORMAT;
-	type = Foi::UNKNOWN_TYPE;
+	fmt = ScFace::UNKNOWN_FORMAT;
+	type = ScFace::UNKNOWN_TYPE;
 	if (ftIOFunc(fts, 0L, reinterpret_cast<FT_Byte *>(buf), 128) == FONT_NO_ERROR) 
 	{
 		if(strncmp(buf,T42_HEAD,strlen(T42_HEAD)) == 0) 
 		{
-			fmt = Foi::TYPE42;
-			type = Foi::TTF;
+			fmt = ScFace::TYPE42;
+			type = ScFace::TTF;
 		}
 		else if(strncmp(buf,T1_HEAD,strlen(T1_HEAD)) == 0 ||
 			    strncmp(buf,T1_ADOBE_HEAD,strlen(T1_ADOBE_HEAD)) == 0) 
 		{
-			fmt = Foi::PFA;
-			type = Foi::TYPE1;
+			fmt = ScFace::PFA;
+			type = ScFace::TYPE1;
 		}
 		else if(strncmp(buf,PSFONT_ADOBE2_HEAD,strlen(PSFONT_ADOBE2_HEAD)) == 0 ||
 			    strncmp(buf,PSFONT_ADOBE21_HEAD,strlen(PSFONT_ADOBE21_HEAD)) == 0 ||
 			    strncmp(buf,PSFONT_ADOBE3_HEAD,strlen(PSFONT_ADOBE3_HEAD)) ==0) 
 		{
 			// Type2(CFF), Type0(Composite/CID), Type 3, Type 14 etc would end here
-			fmt = Foi::PFA;
-			type = Foi::UNKNOWN_TYPE;
+			fmt = ScFace::PFA;
+			type = ScFace::UNKNOWN_TYPE;
 		}
 		else if(buf[0] == '\200' && buf[1] == '\1')
 		{
-			fmt = Foi::PFB;
-			type = Foi::TYPE1;
+			fmt = ScFace::PFB;
+			type = ScFace::TYPE1;
 		}
 		else if(buf[0] == '\0' && buf[1] == '\1' 
 				&& buf[2] == '\0' && buf[3] == '\0')
 		{
-			fmt = Foi::SFNT;
-			type = Foi::TTF;
+			fmt = ScFace::SFNT;
+			type = ScFace::TTF;
 		}
 		else if(strncmp(buf,"true",4) == 0)
 		{
-			fmt = Foi::SFNT;
-			type = Foi::TTF;
+			fmt = ScFace::SFNT;
+			type = ScFace::TTF;
 		}
 		else if(strncmp(buf,"ttcf",4) == 0)
 		{
-			fmt = Foi::TTCF;
-			type = Foi::OTF;
+			fmt = ScFace::TTCF;
+			type = ScFace::OTF;
 		}
 		else if(strncmp(buf,"OTTO",4) == 0)
 		{
-			fmt = Foi::SFNT;
-			type = Foi::OTF;
+			fmt = ScFace::SFNT;
+			type = ScFace::OTF;
 		}
 		// missing: raw CFF
 	}
@@ -671,7 +280,7 @@ void getFontFormat(FT_Face face, Foi::FontFormat & fmt, Foi::FontType & type)
  * Checks face, which must be sfnt based, for the subtype.
  * Possible values: TTF, CFF, OTF
  */
-void getSFontType(FT_Face face, Foi::FontType & type) 
+void getSFontType(FT_Face face, ScFace::FontType & type) 
 {
 	if (FT_IS_SFNT(face)) 
 	{
@@ -681,9 +290,9 @@ void getSFontType(FT_Face face, Foi::FontType & type)
 		bool hasCFF = ! FT_Load_Sfnt_Table(face, TTAG_CFF, 0, NULL,  &ret);
 		hasCFF &= ret > 0;
 		if (hasGlyph)
-			type = Foi::TTF;
+			type = ScFace::TTF;
 		else if (hasCFF)
-			type = Foi::OTF;
+			type = ScFace::OTF;
 		// TODO: CID or no
 	}
 } 
@@ -695,20 +304,27 @@ bool SCFonts::AddScalableFont(QString filename, FT_Library &library, QString Doc
 	bool Subset = false;
 	char *buf[50];
 	QString glyName = "";
-	Foi::FontFormat format;
-	Foi::FontType   type;
+	ScFace::FontFormat format;
+	ScFace::FontType   type;
 	FT_Face         face = NULL;
 	struct testCache foCache;
 	QFileInfo fic(filename);
+	QDateTime lastMod = fic.lastModified();
+	QTime lastModTime = lastMod.time();
+	if (lastModTime.msec() != 0)  //Sometime file time is stored with precision up to msecs
+	{
+		lastModTime.setHMS(lastModTime.hour(), lastModTime.minute(), lastModTime.second());
+		lastMod.setTime(lastModTime);
+	}
 	foCache.isOK = false;
 	foCache.isChecked = true;
-	foCache.lastMod = fic.lastModified();
+	foCache.lastMod = lastMod;
 	if (checkedFonts.count() == 0)
 	{
 		firstRun = true;
-		ScMW->setSplashStatus( QObject::tr("Creating Font Cache") );
+		ScCore->setSplashStatus( QObject::tr("Creating Font Cache") );
 	}
-	bool error = FT_New_Face( library, filename, 0, &face );
+	bool error = FT_New_Face( library, QFile::encodeName(filename), 0, &face );
 	if (error) 
 	{
 		if (face != NULL)
@@ -719,7 +335,7 @@ bool SCFonts::AddScalableFont(QString filename, FT_Library &library, QString Doc
 		return true;
 	}
 	getFontFormat(face, format, type);
-	if (format == Foi::UNKNOWN_FORMAT) 
+	if (format == ScFace::UNKNOWN_FORMAT) 
 	{
 		if (showFontInformation)
 			sDebug(QObject::tr("Failed to load font %1 - font type unknown").arg(filename));
@@ -732,7 +348,7 @@ bool SCFonts::AddScalableFont(QString filename, FT_Library &library, QString Doc
 		if (!checkedFonts.contains(filename))
 		{
 			if (!firstRun)
-				ScMW->setSplashStatus( QObject::tr("New Font found, checking...") );
+				ScCore->setSplashStatus( QObject::tr("New Font found, checking...") );
 			FT_UInt gindex = 0;
 			FT_ULong charcode = FT_Get_First_Char( face, &gindex );
 			while ( gindex != 0 )
@@ -767,9 +383,9 @@ bool SCFonts::AddScalableFont(QString filename, FT_Library &library, QString Doc
 				checkedFonts[filename].isChecked = true;
 				return true;
 			}
-			if (checkedFonts[filename].lastMod != fic.lastModified())
+			if (checkedFonts[filename].lastMod != foCache.lastMod)
 			{
-				ScMW->setSplashStatus( QObject::tr("Modified Font found, checking...") );
+				ScCore->setSplashStatus( QObject::tr("Modified Font found, checking...") );
 				FT_UInt gindex = 0;
 				FT_ULong charcode = FT_Get_First_Char( face, &gindex );
 				while ( gindex != 0 )
@@ -836,80 +452,76 @@ bool SCFonts::AddScalableFont(QString filename, FT_Library &library, QString Doc
 			qpsName = QString(psName);
 		else
 			qpsName = ts;
-		Foi *t = find(ts);
-		if (t)
+		ScFace t;
+		if (contains(ts))
 		{
-			if (t->RealName() != qpsName)
+			t = (*this)[ts];
+			if (t.psName() != qpsName)
 			{
 				alt = " ("+qpsName+")";
 				ts += alt;
 				sty += alt;
 			}
 		}
-		t = find(ts);
-		if (!t)
+		t = (*this)[ts];
+		if (t.isNone())
 		{
 			switch (format) 
 			{
-				case Foi::PFA:
-					t = new Foi_pfa(fam, sty, ts, qpsName, filename, faceindex, true);
-					t->Subset = Subset;
+				case ScFace::PFA:
+					t = ScFace(new ScFace_pfa(fam, sty, "", ts, qpsName, filename, faceindex));
+					t.subset(Subset);
 					break;
-				case Foi::PFB:
-					t = new Foi_pfb(fam, sty, ts, qpsName, filename, faceindex, true);
-					t->Subset = Subset;
+				case ScFace::PFB:
+					t = ScFace(new ScFace_pfb(fam, sty, "", ts, qpsName, filename, faceindex));
+					t.subset(Subset);
 					break;
-				case Foi::SFNT:
-					t = new Foi_ttf(fam, sty, ts, qpsName, filename, faceindex, true);
-					getSFontType(face, t->typeCode);
-					if (t->typeCode == Foi::OTF) 
+				case ScFace::SFNT:
+					t = ScFace(new ScFace_ttf(fam, sty, "", ts, qpsName, filename, faceindex));
+					getSFontType(face, t.m->typeCode);
+					if (t.type() == ScFace::OTF) 
 					{
-						t->isOTF = true;
-						t->Subset = true;
+						t.subset(true);
 					}
 					else
-						t->Subset = Subset;
+						t.subset(Subset);
 					break;
-				case Foi::TTCF:
-					t = new Foi_ttf(fam, sty, ts, qpsName, filename, faceindex, true);
-					t->formatCode = Foi::TTCF;
-					t->typeCode = Foi::TTF;
-					//getSFontType(face, t->typeCode);
-					if (t->typeCode == Foi::OTF) 
+				case ScFace::TTCF:
+					t = ScFace(new ScFace_ttf(fam, sty, "", ts, qpsName, filename, faceindex));
+					t.m->formatCode = ScFace::TTCF;
+					t.m->typeCode = ScFace::TTF;
+					//getSFontType(face, t.m->typeCode);
+					if (t.type() == ScFace::OTF) 
 					{
-						t->isOTF = true;
-						t->Subset = true;
+						t.subset(true);
 					}
 					else
-						t->Subset = Subset;
+						t.subset(Subset);
 					break;
-				case Foi::TYPE42:
-					t = new Foi_ttf(fam, sty, ts, qpsName, filename, faceindex, true);
-					getSFontType(face, t->typeCode);
-					if (t->typeCode == Foi::OTF) 
+				case ScFace::TYPE42:
+					t = ScFace(new ScFace_ttf(fam, sty, "", ts, qpsName, filename, faceindex));
+					getSFontType(face, t.m->typeCode);
+					if (t.type() == ScFace::OTF) 
 					{
-						t->isOTF = true;
-						t->Subset = true;
+						t.subset(true);
 					}
 					else
-						t->Subset = Subset;
+						t.subset(Subset);
 					break;
 				default:
 				/* catching any types not handled above to silence compiler */
 					break;
 			}
 			insert(ts,t);
-			t->HasNames = HasNames;
-			t->EmbedPS = true;
-			t->UseFont = true;
-			t->CharWidth[13] = 0;
-			t->CharWidth[28] = 0;
-			t->CharWidth[9] = 1;
-			if (face->num_glyphs > 512)
-				t->Subset = true;
-			t->PrivateFont = DocName;
+			t.m->hasNames = HasNames;
+			t.embedPs(true);
+			t.usable(true);
+			t.m->status = ScFace::UNKNOWN;
+			if (face->num_glyphs > 2048)
+				t.subset(true);
+			t.m->forDocument = DocName;
 			//setBestEncoding(face); //AV
-			switch (face->charmap? face->charmap->encoding : -1)
+/*			switch (face->charmap? face->charmap->encoding : -1)
 			{
 				case	 FT_ENCODING_ADOBE_STANDARD: 
 					t->FontEnc = QString("StandardEncoding");
@@ -929,8 +541,8 @@ bool SCFonts::AddScalableFont(QString filename, FT_Library &library, QString Doc
 				default:
 					t->FontEnc = QString();
 			}
-			if (showFontInformation)
-				sDebug(QObject::tr("Font %1 loaded from %2(%3)").arg(t->RealName()).arg(filename).arg(faceindex+1));
+*/			if (showFontInformation)
+				sDebug(QObject::tr("Font %1 loaded from %2(%3)").arg(t.psName()).arg(filename).arg(faceindex+1));
 
 /*
 //debug
@@ -946,7 +558,7 @@ bool SCFonts::AddScalableFont(QString filename, FT_Library &library, QString Doc
 		else 
 		{
 			if (showFontInformation)
-				sDebug(QObject::tr("Font %1(%2) is duplicate of %3").arg(filename).arg(faceindex+1).arg(t->fontPath()));
+				sDebug(QObject::tr("Font %1(%2) is duplicate of %3").arg(filename).arg(faceindex+1).arg(t.fontPath()));
 			// this is needed since eg. AppleSymbols will happily return a face for *any* face_index
 			if (faceindex > 0) {
 				break;
@@ -955,7 +567,7 @@ bool SCFonts::AddScalableFont(QString filename, FT_Library &library, QString Doc
 		FT_Done_Face(face);
 		face=NULL;
 		++faceindex;
-		error = FT_New_Face( library, filename, faceindex, &face );
+		error = FT_New_Face( library, QFile::encodeName(filename), faceindex, &face );
 	} //while
 	
 	if (face != 0)
@@ -969,6 +581,62 @@ void SCFonts::removeFont(QString name)
 	updateFontMap();
 }
 
+const ScFace& SCFonts::findFont(const QString& fontname, ScribusDoc *doc)
+{
+	if (fontname.isEmpty())
+		return ScFace::none();
+	
+	PrefsManager* prefsManager = PrefsManager::instance();
+	
+	if (!contains(fontname) || !(*this)[fontname].usable())
+	{
+		QString replFont;
+		if ((!prefsManager->appPrefs.GFontSub.contains(fontname)) || (!(*this)[prefsManager->appPrefs.GFontSub[fontname]].usable()))
+		{
+			replFont = doc ? doc->toolSettings.defFont : prefsManager->appPrefs.toolSettings.defFont;
+		}
+		else
+			replFont = prefsManager->appPrefs.GFontSub[fontname];
+		ScFace repl = (*this)[replFont].mkReplacementFor(fontname, doc ? doc->DocName : QString(""));
+		insert(fontname, repl);
+	}
+	else if ( doc && !doc->UsedFonts.contains(fontname) )
+	{
+		doc->AddFont(fontname, qRound(doc->toolSettings.defSize / 10.0));
+	}
+	return (*this)[fontname];	
+}
+
+
+QMap<QString,QString> SCFonts::getSubstitutions(const QList<QString> skip) const
+{
+	QMap<QString,QString> result;
+	QMap<QString,ScFace>::ConstIterator it;
+	for (it = begin(); it != end(); ++it)
+	{
+		if (it.value().isReplacement() && !skip.contains(it.key()))
+			result[it.key()] = it.value().replacementName();
+	}
+	return result;
+}
+
+
+void SCFonts::setSubstitutions(const QMap<QString,QString>& substitutes, ScribusDoc* doc)
+{
+	QMap<QString,QString>::ConstIterator it;
+	for (it = substitutes.begin(); it != substitutes.end(); ++it)
+	{
+		if (it.key() == it.value())
+			continue;
+		ScFace& font(const_cast<ScFace&>(findFont(it.key(), doc)));
+		if (font.isReplacement())
+		{
+			font.chReplacementTo(const_cast<ScFace&>(findFont(it.value(), doc)), doc->DocName);
+		}
+	}
+}
+
+
 #ifdef HAVE_FONTCONFIG
 // Use Fontconfig to locate and load fonts.
 void SCFonts::AddFontconfigFonts()
@@ -983,7 +651,7 @@ void SCFonts::AddFontconfigFonts()
 									NULL);
 	// The ObjectSet tells FontConfig what information about each match to return.
 	// We currently just need FC_FILE, but other info like font family and style
-	// is availible - see "man fontconfig".
+	// is available - see "man fontconfig".
 	FcObjectSet* os = FcObjectSetBuild (FC_FILE, (char *) 0);
 	// Now ask fontconfig to retrieve info as specified in 'os' about fonts
 	// matching pattern 'pat'.
@@ -1012,8 +680,7 @@ void SCFonts::AddFontconfigFonts()
 	FT_Done_FreeType( library );
 }
 
-#else
-#if !defined(QT_MAC) && !defined(_WIN32)
+#elif defined(Q_WS_X11)
 
 void SCFonts::AddXFontPath()
 {
@@ -1042,11 +709,11 @@ void SCFonts::AddXFontServerPath()
 		}
 	}
 
-	if (fs.open(IO_ReadOnly))
+	if (fs.open(QIODevice::ReadOnly))
 	{
 		QString fsconfig,paths,tmp;
 		QTextStream tsx(&fs);
-		fsconfig = tsx.read();
+		fsconfig = tsx.readAll();
 		fs.close();
 
 		int pos = fsconfig.find("catalogue");
@@ -1060,11 +727,11 @@ void SCFonts::AddXFontServerPath()
 
 		if(pos<0) pos=paths.length();
 		paths = paths.left(pos);
-		paths = paths.simplifyWhiteSpace();
+		paths = paths.simplified();
 		paths.replace(QRegExp(" "), "");
 		paths.replace(QRegExp(","), "\n");
 		paths += "\n";
-		paths = paths.stripWhiteSpace();
+		paths = paths.trimmed();
 
 		pos=-1;
 		do {
@@ -1079,7 +746,6 @@ void SCFonts::AddXFontServerPath()
 	}
 }
 #endif
-#endif
 
 /* Add an extra path to our font search path,
  * allowing a user to have extra fonts installed
@@ -1103,14 +769,14 @@ void SCFonts::ReadCacheList(QString pf)
 	struct testCache foCache;
 	QDomDocument docu("fontcacherc");
 	QFile f(pf + "/checkfonts.xml");
-	if(!f.open(IO_ReadOnly))
+	if(!f.open(QIODevice::ReadOnly))
 		return;
-	ScMW->setSplashStatus( QObject::tr("Reading Font Cache") );
+	ScCore->setSplashStatus( QObject::tr("Reading Font Cache") );
 	QTextStream ts(&f);
-	ts.setEncoding(QTextStream::UnicodeUTF8);
+	ts.setCodec("UTF-8");
 	QString errorMsg;
 	int errorLine = 0, errorColumn = 0;
-	if( !docu.setContent(ts.read(), &errorMsg, &errorLine, &errorColumn) )
+	if( !docu.setContent(ts.readAll(), &errorMsg, &errorLine, &errorColumn) )
 	{
 		f.close();
 		return;
@@ -1143,21 +809,21 @@ void SCFonts::WriteCacheList(QString pf)
 	QMap<QString, testCache>::Iterator it;
 	for (it = checkedFonts.begin(); it != checkedFonts.end(); ++it)
 	{
-		if (it.data().isChecked)
+		if (it.value().isChecked)
 		{
 			QDomElement fosu = docu.createElement("Font");
 			fosu.setAttribute("File",it.key());
-			fosu.setAttribute("Status",static_cast<int>(it.data().isOK));
-			fosu.setAttribute("Modified",it.data().lastMod.toString(Qt::ISODate));
+			fosu.setAttribute("Status",static_cast<int>(it.value().isOK));
+			fosu.setAttribute("Modified",it.value().lastMod.toString(Qt::ISODate));
 			elem.appendChild(fosu);
 		}
 	}
-	ScMW->setSplashStatus( QObject::tr("Writing updated Font Cache") );
+	ScCore->setSplashStatus( QObject::tr("Writing updated Font Cache") );
 	QFile f(pf + "/checkfonts.xml");
-	if(f.open(IO_WriteOnly))
+	if(f.open(QIODevice::WriteOnly))
 	{
 		QTextStream s(&f);
-		s.setEncoding(QTextStream::UnicodeUTF8);
+		s.setCodec("UTF-8");
 		s<<docu.toString();
 		f.close();
 	}
@@ -1169,11 +835,11 @@ void SCFonts::GetFonts(QString pf, bool showFontInfo)
 	showFontInformation=showFontInfo;
 	FontPath.clear();
 	ReadCacheList(pf);
-	ScMW->setSplashStatus( QObject::tr("Searching for Fonts") );
+	ScCore->setSplashStatus( QObject::tr("Searching for Fonts") );
 	AddUserPath(pf);
 	// Search the system paths
 	QStringList ftDirs = ScPaths::getSystemFontDirs();
-	for (unsigned int i = 0; i < ftDirs.count(); i++)
+	for (int i = 0; i < ftDirs.count(); i++)
 		AddScalableFonts( ftDirs[i] );
 	// Search Scribus font path
 	if (!ScPaths::instance().fontDir().isEmpty() && QDir(ScPaths::instance().fontDir()).exists())
@@ -1181,8 +847,9 @@ void SCFonts::GetFonts(QString pf, bool showFontInfo)
 // if fontconfig is there, it does all the work
 #if HAVE_FONTCONFIG
 	// Search fontconfig paths
-	for(QStrListIterator fpi(FontPath) ; fpi.current() ; ++fpi)
-		AddScalableFonts(fpi.current());
+	QStringList::iterator fpi, fpend = FontPath.end();
+	for(fpi = FontPath.begin() ; fpi != fpend; ++fpi) 
+		AddScalableFonts(*fpi);
 	AddFontconfigFonts();
 #else
 // on X11 look there:
@@ -1191,8 +858,9 @@ void SCFonts::GetFonts(QString pf, bool showFontInfo)
 	AddXFontServerPath();
 #endif
 // add user and X11 fonts:
-	for(QStrListIterator fpi(FontPath) ; fpi.current() ; ++fpi) 
-		AddScalableFonts(fpi.current());
+	QStringList::iterator fpi, fpend = FontPath.end();
+	for(fpi = FontPath.begin() ; fpi != fpend; ++fpi) 
+		AddScalableFonts(*fpi);
 #endif
 	updateFontMap();
 	WriteCacheList(pf);
