@@ -4,16 +4,15 @@ to the COPYING file provided with the program. Following this notice may exist
 a copyright and/or license notice that predates the release of Scribus 1.3.2
 for which a new license (GPL+exception) is in place.
 */
-#include <qfile.h>
-#include <qfileinfo.h>
-#include <qobject.h>
+#include <QFile>
+#include <QFileInfo>
+#include <QObject>
 #include <setjmp.h>
 #include "scconfig.h"
 #include "scimgdataloader_jpeg.h"
+#include "colormgmt/sccolormgmtengine.h"
 #include "exif.h"
 
-#include CMS_INC
-
 typedef struct my_error_mgr
 {
 	struct jpeg_error_mgr pub;            /* "public" fields */
@@ -46,7 +45,7 @@ void ScImgDataLoader_JPEG::initSupportedFormatList(void)
 	m_supportedFormats.append( "jpeg" );
 }
 
-void ScImgDataLoader_JPEG::loadEmbeddedProfile(const QString& fn)
+void ScImgDataLoader_JPEG::loadEmbeddedProfile(const QString& fn, int /*page*/)
 {
 	m_embeddedProfile.resize(0);
 	m_profileComponents = 0;
@@ -66,40 +65,42 @@ void ScImgDataLoader_JPEG::loadEmbeddedProfile(const QString& fn)
 		return;
 	}
 	jpeg_create_decompress (&cinfo);
-	if ((infile = fopen (fn.local8Bit(), "rb")) == NULL)
+	if ((infile = fopen (fn.toLocal8Bit(), "rb")) == NULL)
 		return;
 	jpeg_stdio_src(&cinfo, infile);
 	jpeg_save_markers(&cinfo, ICC_MARKER, 0xFFFF);
 	jpeg_read_header(&cinfo, true);
-	//jpeg_start_decompress(&cinfo);
 	unsigned int EmbedLen = 0;
 	unsigned char* EmbedBuffer;
 	if (read_jpeg_marker(ICC_MARKER,&cinfo, &EmbedBuffer, &EmbedLen))
 	{
-		cmsHPROFILE prof = cmsOpenProfileFromMem(EmbedBuffer, EmbedLen);
+		QByteArray profArray = QByteArray((const char*) EmbedBuffer, EmbedLen);
+		ScColorProfile prof  = ScColorMgmtEngine::openProfileFromMem(profArray);
 		if (prof)
 		{
-			if (static_cast<int>(cmsGetColorSpace(prof)) == icSigRgbData)
+			if (prof.colorSpace() == ColorSpace_Rgb)
 				m_profileComponents = 3;
-			if (static_cast<int>(cmsGetColorSpace(prof)) == icSigCmykData)
+			if (prof.colorSpace() == ColorSpace_Cmyk)
 				m_profileComponents = 4;
-			m_embeddedProfile.duplicate((const char*) EmbedBuffer, EmbedLen);
+			if (prof.colorSpace() == ColorSpace_Gray)
+				m_profileComponents = 1;
+			m_embeddedProfile = profArray;
 		}
-		cmsCloseProfile(prof);
 		free(EmbedBuffer);
 	}
-	//(void) jpeg_finish_decompress(&cinfo);
 	fclose (infile);
 	jpeg_destroy_decompress (&cinfo);
 }
 
-void ScImgDataLoader_JPEG::preloadAlphaChannel(const QString& fn, int res)
+bool ScImgDataLoader_JPEG::preloadAlphaChannel(const QString& fn, int /*page*/, int res, bool& hasAlpha)
 {
 	// No support for aplha in jpeg pictures
 	initialize();
+	hasAlpha = false;
+	return true;
 }
 
-bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnail)
+bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int /*page*/, int res, bool thumbnail)
 {
 	bool isCMYK = false;
 	bool fromPS = false;
@@ -115,7 +116,7 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 	infile = NULL;
 
 	initialize();
-	m_imageInfoRecord.type = 0;
+	m_imageInfoRecord.type = ImageTypeJPG;
 	m_imageInfoRecord.exifInfo.thumbnail = QImage();
 	if (setjmp (jerr.setjmp_buffer))
 	{
@@ -125,7 +126,7 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 		return false;
 	}
 	jpeg_create_decompress (&cinfo);
-	if ((infile = fopen (fn.local8Bit(), "rb")) == NULL)
+	if ((infile = fopen (fn.toLocal8Bit(), "rb")) == NULL)
 		return false;
 	jpeg_stdio_src(&cinfo, infile);
 	jpeg_save_markers(&cinfo, ICC_MARKER, 0xFFFF);
@@ -136,12 +137,12 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 	if ((exi) && (ExifInf.exifDataValid))
 	{
 		if (cinfo.output_components == 4)
-			m_imageInfoRecord.colorspace = 1;
+			m_imageInfoRecord.colorspace = ColorSpaceCMYK;
 		else if (cinfo.output_components == 3)
-			m_imageInfoRecord.colorspace = 0;
+			m_imageInfoRecord.colorspace = ColorSpaceRGB;
 		else if (cinfo.output_components == 1)
-			m_imageInfoRecord.colorspace = 2;
-		if ((!ExifInf.isNullThumbnail()) && thumbnail)
+			m_imageInfoRecord.colorspace = ColorSpaceGray;
+		if ((!ExifInf.Thumbnail.isNull()) && thumbnail)
 		{
 			m_image = ExifInf.getThumbnail();
 			m_imageInfoRecord.exifInfo.thumbnail = ExifInf.getThumbnail();
@@ -157,7 +158,7 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 						cc = 255 - qRed(*s);
 						cm = 255 - qGreen(*s);
 						cy = 255 - qBlue(*s);
-						ck = QMIN(QMIN(cc, cm), cy);
+						ck = qMin(qMin(cc, cm), cy);
 						*s = qRgba(cc-ck,cm-ck,cy-ck,ck);
 						s++;
 					}
@@ -202,7 +203,7 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 		m_imageInfoRecord.xres = qRound(xres);
 		m_imageInfoRecord.yres = qRound(yres);
 		m_imageInfoRecord.progressive = jpeg_has_multiple_scans(&cinfo);
-		if ((!ExifInf.isNullThumbnail()) && thumbnail)
+		if ((!ExifInf.Thumbnail.isNull()) && thumbnail)
 		{
 			jpeg_destroy_decompress(&cinfo);
 			fclose(infile);
@@ -216,12 +217,11 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 	unsigned char* EmbedBuffer;
 	if (read_jpeg_marker(ICC_MARKER,&cinfo, &EmbedBuffer, &EmbedLen))
 	{
-		const char *Descriptor;
-		cmsHPROFILE prof = cmsOpenProfileFromMem(EmbedBuffer, EmbedLen);
-		Descriptor = cmsTakeProductDesc(prof);
-		m_embeddedProfile.duplicate((const char*) EmbedBuffer, EmbedLen);
-		m_imageInfoRecord.profileName = QString(Descriptor);
-		m_imageInfoRecord.isEmbedded = true;
+		QByteArray profArray = QByteArray((const char*) EmbedBuffer, EmbedLen);
+		ScColorProfile prof = ScColorMgmtEngine::openProfileFromMem(profArray);
+		m_embeddedProfile   = profArray;
+		m_imageInfoRecord.profileName = prof.productDescription();
+		m_imageInfoRecord.isEmbedded  = true;
 		free(EmbedBuffer);
 	}
 	else
@@ -266,12 +266,12 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 	if (cinfo.output_components == 4)
 	{
 		isCMYK = true;
-		m_imageInfoRecord.colorspace = 1;
+		m_imageInfoRecord.colorspace = ColorSpaceCMYK;
 	}
 	else if (cinfo.output_components == 3)
-		m_imageInfoRecord.colorspace = 0;
+		m_imageInfoRecord.colorspace = ColorSpaceRGB;
 	else if (cinfo.output_components == 1)
-		m_imageInfoRecord.colorspace = 2;
+		m_imageInfoRecord.colorspace = ColorSpaceGray;
 	m_imageInfoRecord.progressive = jpeg_has_multiple_scans(&cinfo);
 
 	if (read_jpeg_marker(PHOTOSHOP_MARKER,&cinfo, &PhotoshopBuffer, &PhotoshopLen) )
@@ -279,19 +279,19 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 		if (PhotoshopLen != 0)
 		{
 			bool savEx = m_imageInfoRecord.exifDataValid;
-			QByteArray arrayPhot(PhotoshopLen);
-			arrayPhot.setRawData((const char*)PhotoshopBuffer,PhotoshopLen);
-			QDataStream strPhot(arrayPhot,IO_ReadOnly);
+			QByteArray arrayPhot(PhotoshopLen, ' ');
+			arrayPhot = QByteArray::fromRawData((const char*)PhotoshopBuffer,PhotoshopLen);
+			QDataStream strPhot(&arrayPhot,QIODevice::ReadOnly);
 			strPhot.setByteOrder( QDataStream::BigEndian );
 			PSDHeader fakeHeader;
 			fakeHeader.width = cinfo.output_width;
 			fakeHeader.height = cinfo.output_height;
 			if (cinfo.output_components == 4)
-				m_imageInfoRecord.colorspace = 1;
+				m_imageInfoRecord.colorspace = ColorSpaceCMYK;
 			else if (cinfo.output_components == 3)
-				m_imageInfoRecord.colorspace = 0;
+				m_imageInfoRecord.colorspace = ColorSpaceRGB;
 			else if (cinfo.output_components == 1)
-				m_imageInfoRecord.colorspace = 2;
+				m_imageInfoRecord.colorspace = ColorSpaceGray;
 			m_imageInfoRecord.progressive = jpeg_has_multiple_scans(&cinfo);
 			parseRessourceData(strPhot, fakeHeader, PhotoshopLen);
 			// Photoshop resolution is more accurate than jpeg header resolution
@@ -310,14 +310,13 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 				m_message = QObject::tr("%1 may be corrupted : missing or wrong resolution tags").arg(qfi.fileName());
 				m_msgType = warningMsg;
 			}
-			if (m_imageInfoRecord.exifDataValid && thumbnail)
+			if (m_imageInfoRecord.exifDataValid && !m_imageInfoRecord.exifInfo.thumbnail.isNull() && thumbnail)
 			{
-				m_image.create(m_imageInfoRecord.exifInfo.width, m_imageInfoRecord.exifInfo.height, 32 );
+				m_image = QImage(m_imageInfoRecord.exifInfo.width, m_imageInfoRecord.exifInfo.height, QImage::Format_ARGB32 );
 				m_imageInfoRecord.exifInfo.width = cinfo.output_width;
 				m_imageInfoRecord.exifInfo.height = cinfo.output_height;
 				if (cinfo.output_components == 4)
 				{
-					m_image.setAlphaBuffer(false);
 					QRgb *d;
 					QRgb *s;
 					unsigned char cc, cm, cy, ck;
@@ -330,7 +329,7 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 							cc = 255 - qRed(*s);
 							cm = 255 - qGreen(*s);
 							cy = 255 - qBlue(*s);
-							ck = QMIN(QMIN(cc, cm), cy);
+							ck = qMin(qMin(cc, cm), cy);
 							*d = qRgba(cc-ck,cm-ck,cy-ck,ck);
 							s++;
 							d++;
@@ -341,9 +340,9 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 					m_image = m_imageInfoRecord.exifInfo.thumbnail.copy();
 			}
 			m_imageInfoRecord.valid = (m_imageInfoRecord.PDSpathData.size())>0?true:false; // The only interest is vectormask
-			arrayPhot.resetRawData((const char*)PhotoshopBuffer,PhotoshopLen);
+			arrayPhot.clear();
 			free( PhotoshopBuffer );
-			if (m_imageInfoRecord.exifDataValid && thumbnail)
+			if (m_imageInfoRecord.exifDataValid && !m_imageInfoRecord.exifInfo.thumbnail.isNull() && thumbnail)
 			{
 				jpeg_destroy_decompress(&cinfo);
 				fclose(infile);
@@ -355,18 +354,23 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 		}
 	}
 	if ( cinfo.output_components == 3 || cinfo.output_components == 4)
-		m_image.create( cinfo.output_width, cinfo.output_height, 32 );
+		m_image = QImage( cinfo.output_width, cinfo.output_height, QImage::Format_ARGB32 );
 	else if ( cinfo.output_components == 1 )
 	{
-		m_image.create( cinfo.output_width, cinfo.output_height, 8, 256 );
+		m_image = QImage( cinfo.output_width, cinfo.output_height, QImage::Format_Indexed8 );
+		m_image.setNumColors(256);
 		for (int i=0; i<256; i++)
 			m_image.setColor(i, qRgb(i,i,i));
 	}
 	if (!m_image.isNull())
 	{
-		uchar** lines = m_image.jumpTable();
+		uchar* data = m_image.bits();
+		int bpl = m_image.bytesPerLine();
 		while (cinfo.output_scanline < cinfo.output_height)
-			(void) jpeg_read_scanlines(&cinfo, lines + cinfo.output_scanline, cinfo.output_height);
+		{
+			uchar *d = data + cinfo.output_scanline * bpl;
+			(void) jpeg_read_scanlines(&cinfo, &d, 1);
+		}
 		if ( cinfo.output_components == 3 )
 		{
 			uchar *in;
@@ -446,8 +450,8 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 			isCMYK = false;
 		if ( cinfo.output_components == 1 )
 		{
-			QImage tmpImg = m_image.convertDepth(32);
-			m_image.create( cinfo.output_width, cinfo.output_height, 32 );
+			QImage tmpImg = m_image.convertToFormat(QImage::Format_ARGB32);
+			m_image = QImage( cinfo.output_width, cinfo.output_height, QImage::Format_ARGB32 );
 			QRgb *s;
 			QRgb *d;
 			for( int yi=0; yi < tmpImg.height(); ++yi )
@@ -462,7 +466,6 @@ bool ScImgDataLoader_JPEG::loadPicture(const QString& fn, int res, bool thumbnai
 				}
 			}
 		}
-		m_image.setAlphaBuffer(true);
 	}
 	(void) jpeg_finish_decompress(&cinfo);
 	fclose (infile);
