@@ -5,14 +5,15 @@ a copyright and/or license notice that predates the release of Scribus 1.3.2
 for which a new license (GPL+exception) is in place.
 */
 #include "scpaths.h"
-#include <qapplication.h>
-#include <qdir.h>
+#include <QApplication>
+#include <QDebug>
+#include <QDir>
 
 #include "scconfig.h"
 
 // On Qt/Mac we need CoreFoundation to discover the location
 // of the app bundle.
-#ifdef BUILD_MAC_BUNDLE
+#ifdef Q_WS_MAC
 #include <CoreFoundation/CoreFoundation.h>
 #endif
 
@@ -60,7 +61,9 @@ ScPaths::ScPaths() :
 {
 // On MacOS/X, override the compile-time settings with a location
 // obtained from the system.
-#ifdef BUILD_MAC_BUNDLE
+#ifdef Q_WS_MAC
+	QString pathPtr(bundleDir());
+	/*
 	// Set up the various app paths to look inside the app bundle
 	CFURLRef pluginRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
 	CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef,
@@ -88,30 +91,30 @@ ScPaths::ScPaths() :
 		p -= 9;
 		*p = '\0';
 	}
-
-	qDebug(QString("scpaths: bundle at %1:").arg(pathPtr));
+*/
+	qDebug() << QString("scpaths: bundle at %1:").arg(pathPtr);
 	m_shareDir = QString("%1/Contents/share/scribus/").arg(pathPtr);
-	m_docDir = QString("%1/Contents/share/scribus/doc/").arg(pathPtr);
+	m_docDir = QString("%1/Contents/share/doc/scribus/").arg(pathPtr);
 	m_iconDir = QString("%1/Contents/share/scribus/icons/").arg(pathPtr);
 	m_sampleScriptDir = QString("%1/Contents/share/scribus/samples/").arg(pathPtr);
 	m_scriptDir = QString("%1/Contents/share/scribus/scripts/").arg(pathPtr);
 	m_templateDir = QString("%1/Contents/share/scribus/templates/").arg(pathPtr);
 	m_libDir = QString("%1/Contents/lib/scribus/").arg(pathPtr);
 	m_pluginDir = QString("%1/Contents/lib/scribus/plugins/").arg(pathPtr);
-	QApplication::setLibraryPaths(QString("%1/Contents/lib/qtplugins/").arg(pathPtr));
-	CFRelease(pluginRef);
-	CFRelease(macPath);
+	QApplication::setLibraryPaths(QStringList(QString("%1/Contents/lib/qtplugins/").arg(pathPtr)));
+//	CFRelease(pluginRef);
+//	CFRelease(macPath);
 
 	// on OSX this goes to the sys console, so user only sees it when they care -- AV
-	qDebug(QString("scpaths: doc dir=%1").arg(m_docDir));
-	qDebug(QString("scpaths: icon dir=%1").arg(m_iconDir));
-	qDebug(QString("scpaths: font dir=%1").arg(m_fontDir));
-	qDebug(QString("scpaths: sample dir=%1").arg(m_sampleScriptDir));
-	qDebug(QString("scpaths: script dir=%1").arg(m_scriptDir));
-	qDebug(QString("scpaths: template dir=%1").arg(m_templateDir));
-	qDebug(QString("scpaths: lib dir=%1").arg(m_libDir));
-	qDebug(QString("scpaths: plugin dir=%1").arg(m_pluginDir));
-	qDebug(QString("scpaths: qtplugins=%1").arg(QApplication::libraryPaths().join(":")));
+	qDebug() << QString("scpaths: doc dir=%1").arg(m_docDir);
+	qDebug() << QString("scpaths: icon dir=%1").arg(m_iconDir);
+	qDebug() << QString("scpaths: font dir=%1").arg(m_fontDir);
+	qDebug() << QString("scpaths: sample dir=%1").arg(m_sampleScriptDir);
+	qDebug() << QString("scpaths: script dir=%1").arg(m_scriptDir);
+	qDebug() << QString("scpaths: template dir=%1").arg(m_templateDir);
+	qDebug() << QString("scpaths: lib dir=%1").arg(m_libDir);
+	qDebug() << QString("scpaths: plugin dir=%1").arg(m_pluginDir);
+	qDebug() << QString("scpaths: qtplugins=%1").arg(QApplication::libraryPaths().join(":"));
 
 #elif defined(_WIN32)
 	QString appPath = qApp->applicationDirPath();
@@ -124,7 +127,7 @@ ScPaths::ScPaths() :
 	m_templateDir = QString("%1/share/templates/").arg(appPath);
 	m_libDir = QString("%1/libs/").arg(appPath);
 	m_pluginDir = QString("%1/plugins/").arg(appPath);
-	QApplication::setLibraryPaths(QString("%1/qtplugins/").arg(appPath));
+	QApplication::setLibraryPaths( QStringList(QString("%1/qtplugins/").arg(appPath)) );
 #endif
 	
 // 	if(!m_shareDir.endsWith("/"))        m_shareDir += "/";
@@ -140,11 +143,58 @@ ScPaths::ScPaths() :
 
 ScPaths::~ScPaths() {};
 
-QString ScPaths::separatorsToSlashes(const QString& path)
+QString ScPaths::bundleDir(void) const
 {
-	QString qtPath(path);
-	qtPath.replace( QDir::separator(), '/' );
-	return qtPath;
+	// On MacOS/X, override the compile-time settings with a location
+	// obtained from the system.
+#ifdef Q_WS_MAC
+	// Set up the various app paths to look inside the app bundle
+	CFURLRef pluginRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
+	CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, kCFURLPOSIXPathStyle);
+	const char *pathPtr = CFStringGetCStringPtr(macPath, CFStringGetSystemEncoding());
+	if (pathPtr!=NULL && strlen(pathPtr)>0)
+	{
+		// make sure we get the Scribus.app directory, not some subdir
+		// strip trailing '/':
+		qDebug("Path = %s", pathPtr);
+		char *p = const_cast<char*>(pathPtr + strlen(pathPtr) - 1);
+		while (*p == '/')
+			--p;
+		++p;
+		*p = '\0';
+		if (strcmp("/bin", p-4) == 0) {
+			p -= 4;
+			*p = '\0';
+		}
+		if (strcmp("/MacOS", p-6) == 0) {
+			p -= 6;
+			*p = '\0';
+		}
+		if (strcmp("/Contents", p-9) == 0) {
+			p -= 9;
+			*p = '\0';
+		}
+		CFRelease(pluginRef);
+		CFRelease(macPath);
+		return QString("%1").arg(pathPtr);
+	}
+	else
+	{
+		char buf[2048];
+		CFStringGetCString (macPath, buf, 2048, kCFStringEncodingUTF8);
+		QString q_pathPtr=QString::fromUtf8(buf);
+		if (q_pathPtr.endsWith("/bin"))
+			q_pathPtr.chop(4);
+		if (q_pathPtr.endsWith("/MacOS"))
+			q_pathPtr.chop(6);
+		if (q_pathPtr.endsWith("/Contents"))
+			q_pathPtr.chop(9);
+		CFRelease(pluginRef);
+		CFRelease(macPath);
+		return q_pathPtr;
+	}
+#endif
+	return QString::null;
 }
 
 const QString&  ScPaths::docDir() const
@@ -192,11 +242,21 @@ const QString&  ScPaths::shareDir() const
 	return m_shareDir;
 }
 
+QString ScPaths::translationDir() const
+{
+	return (m_shareDir + "translations/");
+}
+
+QString ScPaths::dictDir() const
+{
+	return(m_shareDir + "dicts/");
+}
+
 QStringList ScPaths::getSystemFontDirs(void)
 {
 	QStringList fontDirs;
 #ifdef Q_OS_MAC
-	fontDirs.append(QDir::homeDirPath() + "/Library/Fonts/");
+	fontDirs.append(QDir::homePath() + "/Library/Fonts/");
 	fontDirs.append("/Library/Fonts/");
 	fontDirs.append("/Network/Library/Fonts/");
 	fontDirs.append("/System/Library/Fonts/");
@@ -210,23 +270,24 @@ QStringList ScPaths::getSystemProfilesDirs(void)
 {
 	QStringList iccProfDirs;
 #ifdef Q_OS_MAC
-	iccProfDirs.append(QDir::homeDirPath()+"/Library/ColorSync/Profiles/");
+	iccProfDirs.append(QDir::homePath()+"/Library/ColorSync/Profiles/");
 	iccProfDirs.append("/System/Library/ColorSync/Profiles/");
 	iccProfDirs.append("/Library/ColorSync/Profiles/");
 #elif defined(Q_WS_X11)
-	iccProfDirs.append(QDir::homeDirPath()+"/color/icc/");
-	iccProfDirs.append(QDir::homeDirPath()+"/.color/icc/");
+	iccProfDirs.append(QDir::homePath()+"/color/icc/");
+	iccProfDirs.append(QDir::homePath()+"/.color/icc/");
 	iccProfDirs.append("/usr/share/color/icc/");
 	iccProfDirs.append("/usr/local/share/color/icc/");
 #elif defined(_WIN32)
 	// On Windows it's more complicated, profiles location depends on OS version
-	char sysDir[MAX_PATH + 1];
+	WCHAR sysDir[MAX_PATH + 1];
 	OSVERSIONINFO osVersion;
 	ZeroMemory( &osVersion, sizeof(OSVERSIONINFO));
 	osVersion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO); // Necessary for GetVersionEx to succeed
 	GetVersionEx(&osVersion);  // Get Windows version infos
-	GetSystemDirectory( sysDir, MAX_PATH ); // getSpecialDir(CSIDL_SYSTEM) fails on Win9x
-	QString winSysDir = QString(sysDir).replace('\\','/');
+	GetSystemDirectoryW( sysDir, MAX_PATH ); // getSpecialDir(CSIDL_SYSTEM) fails on Win9x
+	QString winSysDir = QString::fromUtf16((const ushort*) sysDir);
+	winSysDir = winSysDir.replace('\\','/');
 	if( osVersion.dwPlatformId == VER_PLATFORM_WIN32_NT	) // Windows NT/2k/XP
 	{
 		if( osVersion.dwMajorVersion >= 5 ) // for 2k and XP dwMajorVersion == 5 
@@ -245,11 +306,11 @@ QStringList ScPaths::getSystemCreateSwatchesDirs(void)
 {
 	QStringList createDirs;
 #ifdef Q_OS_MAC
-	createDirs.append(QDir::homeDirPath()+"/create/swatches/");
-	createDirs.append(QDir::homeDirPath()+"/.create/swatches/");
+	createDirs.append(QDir::homePath()+"/create/swatches/");
+	createDirs.append(QDir::homePath()+"/.create/swatches/");
 #elif defined(Q_WS_X11)
-	createDirs.append(QDir::homeDirPath()+"/create/swatches/");
-	createDirs.append(QDir::homeDirPath()+"/.create/swatches/");
+	createDirs.append(QDir::homePath()+"/create/swatches/");
+	createDirs.append(QDir::homePath()+"/.create/swatches/");
 	createDirs.append("/usr/share/create/swatches/");
 	createDirs.append("/usr/local/share/create/swatches/");
 #elif defined(_WIN32)
@@ -272,9 +333,22 @@ QString ScPaths::getApplicationDataDir(void)
 #if defined(_WIN32)
 	QString appData = getSpecialDir(CSIDL_APPDATA);
 	if (QDir(appData).exists())
+#ifdef APPLICATION_DATA_DIR
+		return (appData + "/" + APPLICATION_DATA_DIR + "/");
+#else
 		return (appData + "/Scribus/");
 #endif
-	return (QDir::homeDirPath() + "/.scribus/");
+#endif
+
+#ifdef APPLICATION_DATA_DIR
+	return QDir::homePath() + "/" + APPLICATION_DATA_DIR + "/";
+#else
+	#ifdef Q_OS_MAC
+		return (QDir::homePath() + "/Library/Preferences/Scribus/");
+	#else
+		return (QDir::homePath() + "/.scribus/");
+	#endif
+#endif
 }
 
 QString ScPaths::getUserDocumentDir(void)
@@ -284,7 +358,7 @@ QString ScPaths::getUserDocumentDir(void)
 	if	(QDir(userDocs).exists())
 		return userDocs;
 #endif
-	return (QDir::homeDirPath() + "/");
+	return (QDir::homePath() + "/");
 }
 
 QString ScPaths::getTempFileDir(void)
@@ -295,7 +369,7 @@ QString ScPaths::getTempFileDir(void)
 	DWORD result = GetTempPathW(1024, wTempPath);
 	if ( result )
 	{
-		tempPath = QString::fromUcs2((const unsigned short*) wTempPath);
+		tempPath = QString::fromUtf16((const unsigned short*) wTempPath);
 		tempPath.replace( '\\', '/' );
 		tempPath += "/";
 		// GetTempPath may return Windows directory, better not use this one
@@ -303,20 +377,18 @@ QString ScPaths::getTempFileDir(void)
 		if (QDir(tempPath).exists() && tempPath != getSpecialDir(CSIDL_WINDOWS))
 			return tempPath;
 	}
-	return getApplicationDataDir();
-#else
-	return (QDir::homeDirPath() + "/.scribus/");
 #endif
+	return getApplicationDataDir();
 }
 
 QString ScPaths::getSpecialDir(int folder)
 {
 	QString qstr;
 #if defined(_WIN32)
-	char dir[256];
-	if ( SHGetSpecialFolderPath(NULL, dir, folder , false) )
+	WCHAR dir[256];
+	if ( SHGetSpecialFolderPathW(NULL, dir, folder , false) )
 	{
-		qstr = dir;
+		qstr = QString::fromUtf16((const unsigned short*) dir);
 		if( !qstr.endsWith("\\") )
 			qstr += "\\";
 		qstr.replace( '\\', '/' );
