@@ -24,40 +24,67 @@ for which a new license (GPL+exception) is in place.
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 #include "tabkeyboardshortcutswidget.h"
-#include "tabkeyboardshortcutswidget.moc"
 
-#include <qcombobox.h>
-#include <qdir.h>
-#include <qdom.h>
-#include <qfile.h>
-#include <qfileinfo.h>
-#include <qfiledialog.h>
-#include <qinputdialog.h>
-#include <qkeysequence.h>
-#include <qlabel.h>
-#include <qlistview.h>
-#include <qmessagebox.h>
-#include <qpushbutton.h>
-#include <qradiobutton.h>
-#include <qstring.h>
-#include <qstringlist.h>
+#include <QComboBox>
+#include <QDataStream>
+#include <QDir>
+#include <QDomDocument>
+#include <QFile>
+#include <QFileInfo>
+#include <QFileDialog>
+#include <QInputDialog>
+#include <QKeySequence>
+#include <QLabel>
+#include <QListWidget>
+#include <QMessageBox>
+#include <QPushButton>
+#include <QRadioButton>
+#include <QTextStream>
+#include <QList>
+#include <QEvent>
 
 #include "actionmanager.h"
 #include "commonstrings.h"
+#include "pluginmanager.h"
 #include "prefsmanager.h"
 #include "prefsfile.h"
 #include "prefscontext.h"
 #include "scpaths.h"
+#include "scplugin.h"
 #include "scraction.h"
 #include "util.h"
+#include "util_icon.h"
 
 TabKeyboardShortcutsWidget::TabKeyboardShortcutsWidget(QMap<QString, Keys> oldKeyMap, QWidget *parent, const char *name)
-    :TabKeyboardShortcutsWidgetBase(parent, name)
+    :QWidget(parent)
 {
-	ActionManager::createDefaultMenus();
-	ActionManager::createDefaultNonMenuActions();
+	setupUi(this);
+	setObjectName(name);
+// 	ActionManager::createDefaultMenus();
+// 	ActionManager::createDefaultNonMenuActions();
 	defMenus=ActionManager::defaultMenus();
 	defNonMenuActions=ActionManager::defaultNonMenuActions();
+
+	QVector< QPair<QString, QStringList> >::Iterator itnmenua = defNonMenuActions->begin();
+	PluginManager& pluginManager(PluginManager::instance());
+	QStringList pluginNames(pluginManager.pluginNames(false));
+	ScPlugin* plugin;
+	ScActionPlugin* ixplug;
+	QString pName;
+	for (int i = 0; i < pluginNames.count(); ++i)
+	{
+		pName = pluginNames.at(i);
+		plugin = pluginManager.getPlugin(pName, true);
+		Q_ASSERT(plugin); // all the returned names should represent loaded plugins
+		if (plugin->inherits("ScActionPlugin"))
+		{
+			ixplug = dynamic_cast<ScActionPlugin*>(plugin);
+			Q_ASSERT(ixplug);
+			ScActionPlugin::ActionInfo ai(ixplug->actionInfo());
+			itnmenua->second << ai.name;
+		}
+	}
+
 	Q_CHECK_PTR(defMenus);
 	lviToActionMap.clear();
 	lviToMenuMap.clear();
@@ -73,13 +100,10 @@ TabKeyboardShortcutsWidget::TabKeyboardShortcutsWidget(QMap<QString, Keys> oldKe
 	keyDisplay->setMinimumWidth(fontMetrics().width("CTRL+ALT+SHIFT+W"));
 	keyDisplay->setText("");
 
-	keyTable->setSorting(-1);
-	// switched off as it's called in main prefs classes - PV
-	//restoreDefaults();
-
-	clearSearchButton->setPixmap(loadIcon("clear_right.png"));
+	clearSearchButton->setIcon(loadIcon("clear_right.png"));
 	// signals and slots connections
-	connect( keyTable, SIGNAL(clicked(QListViewItem*)), this, SLOT(dispKey(QListViewItem*)));
+	connect( keyTable, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)),
+			 this, SLOT(dispKey(QTreeWidgetItem *, QTreeWidgetItem *)));
 	connect( noKey, SIGNAL(clicked()), this, SLOT(setNoKey()));
 	connect( setKeyButton, SIGNAL(clicked()), this, SLOT(setKeyText()));
 	connect( loadSetButton, SIGNAL(clicked()), this, SLOT(loadKeySetFile()));
@@ -93,7 +117,7 @@ TabKeyboardShortcutsWidget::TabKeyboardShortcutsWidget(QMap<QString, Keys> oldKe
 void TabKeyboardShortcutsWidget::restoreDefaults()
 {
 	loadableSets->clear();
-	loadableSets->insertStringList(scanForSets());
+	loadableSets->addItems(scanForSets());
 	insertActions();
 	dispKey(0);
 }
@@ -119,15 +143,15 @@ void TabKeyboardShortcutsWidget::setNoKey()
 
 void TabKeyboardShortcutsWidget::loadKeySetFile()
 {
-	if (keySetList[loadableSets->currentText()])
+	if (keySetList.contains(loadableSets->currentText()))
 		importKeySet(keySetList[loadableSets->currentText()]);
 }
 
 void TabKeyboardShortcutsWidget::importKeySetFile()
 {
 	PrefsContext* dirs = PrefsManager::instance()->prefsFile->getContext("dirs");
-	QString currentPath = dirs->get("keymapprefs_import", ".");
-	QString s = QFileDialog::getOpenFileName(currentPath, tr("Key Set XML Files (*.ksxml)"), this, "load open file dialog", "Choose a file to open" );
+	QString currentPath = dirs->get("keymapprefs_import", ScPaths::instance().shareDir() + "keysets/");
+	QString s = QFileDialog::getOpenFileName(this, tr("Select a Key set file to read"), currentPath, tr("Key Set XML Files (*.xml)"));
 	if (!s.isEmpty())
 		importKeySet(s);
 }
@@ -135,7 +159,7 @@ void TabKeyboardShortcutsWidget::exportKeySetFile()
 {   
 	PrefsContext* dirs = PrefsManager::instance()->prefsFile->getContext("dirs");
 	QString currentPath= dirs->get("keymapprefs_export", ".");
-	QString s = QFileDialog::getSaveFileName(currentPath, tr("Key Set XML Files (*.ksxml)"), this, "save open file dialog", "Choose a file to save" );
+	QString s = QFileDialog::getSaveFileName(this, tr("Select a Key set file to save to"), currentPath, tr("Key Set XML Files (*.xml)") );
 	if (!s.isEmpty())
 		exportKeySet(s);
 }
@@ -148,21 +172,21 @@ void TabKeyboardShortcutsWidget::importKeySet(QString filename)
 	{
 		//import the file into qdomdoc
 		QDomDocument doc( "keymapentries" );
-		QFile file( filename );
-		if ( !file.open( IO_ReadOnly ) )
+		QFile file1( filename );
+		if ( !file1.open( QIODevice::ReadOnly ) )
 			return;
-		QTextStream ts(&file);
-		ts.setEncoding(QTextStream::UnicodeUTF8);
+		QTextStream ts(&file1);
+		ts.setCodec("UTF-8");
 		QString errorMsg;
 		int eline;
 		int ecol;
-		if ( !doc.setContent( ts.read(), &errorMsg, &eline, &ecol )) 
+		if ( !doc.setContent( ts.readAll(), &errorMsg, &eline, &ecol )) 
 		{
-			qDebug("%s", QString("Could not open key set file: %1\nError:%2 at line: %3, row: %4").arg(filename).arg(errorMsg).arg(eline).arg(ecol).ascii());
-			file.close();
+			qDebug("%s", QString("Could not open key set file: %1\nError:%2 at line: %3, row: %4").arg(filename).arg(errorMsg).arg(eline).arg(ecol).toAscii().constData());
+			file1.close();
 			return;
 		}
-		file.close();
+		file1.close();
 		//load the file now
 		QDomElement docElem = doc.documentElement();
 		if (docElem.tagName()=="shortcutset" && docElem.hasAttribute("name"))
@@ -171,7 +195,7 @@ void TabKeyboardShortcutsWidget::importKeySet(QString filename)
 
 			//clear current menu entries
 			for (QMap<QString,Keys>::Iterator it=keyMap.begin(); it!=keyMap.end(); ++it)
-				it.data().keySequence = QKeySequence();
+				it.value().keySequence = QKeySequence();
 			
 			//load in new set
 			QDomNode n = docElem.firstChild();
@@ -199,14 +223,14 @@ bool TabKeyboardShortcutsWidget::exportKeySet(QString filename)
 {
 	QFileInfo fi = QFileInfo(filename);
 	QString exportFileName;
-	if (filename.endsWith(".ksxml"))
+	if (filename.endsWith(".xml"))
 		exportFileName = filename;
 	else
-		exportFileName = filename+".ksxml";
+		exportFileName = filename+".xml";
 	if (overwrite(this, exportFileName))
 	{
 		bool ok;
-		QString setName = QInputDialog::getText("Export Keyboard Shortcuts to File", "Enter the name of the shortcut set:", QLineEdit::Normal, QString::null, &ok, this );
+		QString setName = QInputDialog::getText(this, tr("Export Keyboard Shortcuts to File"), tr("Enter the name of the shortcut set:"), QLineEdit::Normal, QString::null, &ok);
 		if (!( ok && !setName.isEmpty()) ) 
 			return false;
 		
@@ -217,22 +241,21 @@ bool TabKeyboardShortcutsWidget::exportKeySet(QString filename)
 		QMap<QString,Keys>::Iterator itEnd=keyMap.end();
 		for (QMap<QString,Keys>::Iterator it=keyMap.begin(); it!=itEnd; ++it)
 		{
-			if (it.data().keySequence.isEmpty() && it.key().isEmpty())
+			if (it.value().keySequence.isEmpty() && it.key().isEmpty())
 				continue;
 			QDomElement function_shortcut=doc.createElement("function");
 			function_shortcut.setAttribute("name",it.key());
-			function_shortcut.setAttribute("shortcut",getKeyText(it.data().keySequence));
+			function_shortcut.setAttribute("shortcut",getKeyText(it.value().keySequence));
 			keySetElement.appendChild(function_shortcut);
 		}
 		QFile f(filename);
-		if(!f.open(IO_WriteOnly))
+		if(!f.open(QIODevice::WriteOnly))
 			return false;
-		QTextStream s(&f);
-		s.setEncoding(QTextStream::UnicodeUTF8);
-		QString xmltag("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
-		s.writeRawBytes(xmltag, xmltag.length());
-		QString xmldoc = doc.toString(4);
-		s.writeRawBytes(xmldoc, xmldoc.length());
+		QDataStream s(&f);
+		QByteArray xmltag("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+		s.writeRawData(xmltag.data(), xmltag.length());
+		QByteArray xmldoc = doc.toByteArray(4);
+		s.writeRawData(xmldoc, xmldoc.length());
 		f.close();
 	}
 	return true;
@@ -240,17 +263,17 @@ bool TabKeyboardShortcutsWidget::exportKeySet(QString filename)
 
 void TabKeyboardShortcutsWidget::resetKeySet()
 {
-	QString location=ScPaths::instance().libDir();
-	QString defaultKeySetFileName=QDir::convertSeparators(location+"keysets/scribus13.ksxml");
+	QString location=ScPaths::instance().shareDir();
+	QString defaultKeySetFileName=QDir::convertSeparators(location+"keysets/scribus13.xml");
 	importKeySet(defaultKeySetFileName);	
 }
 
 QStringList TabKeyboardShortcutsWidget::scanForSets()
 {
 	keySetList.clear();
-	QString location=ScPaths::instance().libDir();
+	QString location=ScPaths::instance().shareDir();
 	QString keySetLocation=QDir::convertSeparators(location+"keysets/");
-	QDir keySetsDir(keySetLocation, "*.ksxml", QDir::Name, QDir::Files | QDir::NoSymLinks);
+	QDir keySetsDir(keySetLocation, "*.xml", QDir::Name, QDir::Files | QDir::NoSymLinks);
 	if ((keySetsDir.exists()) && (keySetsDir.count() != 0))
 	{
 		QStringList appNames;
@@ -260,7 +283,7 @@ QStringList TabKeyboardShortcutsWidget::scanForSets()
 			
 			QDomDocument doc( "keymapentries" );
 			QFile file( filename );
-			if ( !file.open( IO_ReadOnly ) )
+			if ( !file.open( QIODevice::ReadOnly ) )
 				continue;
 			QString errorMsg;
 			int eline;
@@ -268,7 +291,7 @@ QStringList TabKeyboardShortcutsWidget::scanForSets()
 			
 			if ( !doc.setContent( &file, &errorMsg, &eline, &ecol )) 
 			{
-				qDebug("%s", QString("Could not open key set file: %1\nError:%2 at line: %3, row: %4").arg(keySetsDir[fileCounter]).arg(errorMsg).arg(eline).arg(ecol).ascii());
+				qDebug("%s", QString("Could not open key set file: %1\nError:%2 at line: %3, row: %4").arg(keySetsDir[fileCounter]).arg(errorMsg).arg(eline).arg(ecol).toAscii().constData());
 				file.close();
 				continue;
 			}
@@ -284,7 +307,7 @@ QStringList TabKeyboardShortcutsWidget::scanForSets()
 		}
 		return QStringList(appNames);
 	}
-	return NULL;
+	return QStringList();
 }
 
 QString TabKeyboardShortcutsWidget::getKeyText(int KeyC)
@@ -307,7 +330,12 @@ QString TabKeyboardShortcutsWidget::getKeyText(int KeyC)
 
 void TabKeyboardShortcutsWidget::setKeyText()
 {
-	if (setKeyButton->isOn())
+	if (keyTable->currentItem()==0)
+	{
+		setKeyButton->setChecked(false);
+		return;
+	}
+	if (setKeyButton->isChecked())
 	{
 		keyCode = 0;
 		Part0 = "";
@@ -327,23 +355,23 @@ void TabKeyboardShortcutsWidget::insertActions()
 	lviToMenuMap.clear();
 	keyTable->clear();
 	bool first, firstMenu=true;
-	QListViewItem *currLVI = NULL;
-	QListViewItem *currMenuLVI = NULL;
-	QListViewItem *prevLVI = NULL;
-	QListViewItem *prevMenuLVI = NULL;
-	for (QValueVector< QPair<QString, QStringList> >::Iterator itmenu = defMenus->begin(); itmenu != defMenus->end(); ++itmenu )
+	QTreeWidgetItem *currLVI = NULL;
+	QTreeWidgetItem *currMenuLVI = NULL;
+	QTreeWidgetItem *prevLVI = NULL;
+	QTreeWidgetItem *prevMenuLVI = NULL;
+	for (QVector< QPair<QString, QStringList> >::Iterator itmenu = defMenus->begin(); itmenu != defMenus->end(); ++itmenu )
 	{
 		if (firstMenu)
 		{
-			currMenuLVI=new QListViewItem(keyTable);
-			firstMenu=false;
+			currMenuLVI = new QTreeWidgetItem(keyTable);
+			firstMenu = false;
 		}
 		else
-			currMenuLVI=new QListViewItem(keyTable, prevMenuLVI);
+			currMenuLVI = new QTreeWidgetItem(keyTable, prevMenuLVI);
 		Q_CHECK_PTR(currMenuLVI);
 		lviToMenuMap.append(currMenuLVI);
 		currMenuLVI->setText(0, itmenu->first);
-		currMenuLVI->setOpen(true);
+		currMenuLVI->setExpanded(true);
 		prevMenuLVI=currMenuLVI;
 		first=true;
 		currLVI=0;
@@ -352,11 +380,11 @@ void TabKeyboardShortcutsWidget::insertActions()
 		{
 			if (first)
 			{
-				currLVI=new QListViewItem(currMenuLVI);
-				first=false;
+				currLVI = new QTreeWidgetItem(currMenuLVI);
+				first = false;
 			}
 			else
-				currLVI=new QListViewItem(currMenuLVI, prevLVI);
+				currLVI = new QTreeWidgetItem(currMenuLVI, prevLVI);
 			Q_CHECK_PTR(currLVI);
 			lviToActionMap.insert(currLVI, *it);
 			currLVI->setText(0, keyMap[*it].cleanMenuText);
@@ -365,19 +393,19 @@ void TabKeyboardShortcutsWidget::insertActions()
 		}
 	}
 	//Non menu actions
-	for (QValueVector< QPair<QString, QStringList> >::Iterator itmenu = defNonMenuActions->begin(); itmenu != defNonMenuActions->end(); ++itmenu )
+	for (QVector< QPair<QString, QStringList> >::Iterator itmenu = defNonMenuActions->begin(); itmenu != defNonMenuActions->end(); ++itmenu )
 	{
 		if (firstMenu)
 		{
-			currMenuLVI=new QListViewItem(keyTable);
-			firstMenu=false;
+			currMenuLVI = new QTreeWidgetItem(keyTable);
+			firstMenu = false;
 		}
 		else
-			currMenuLVI=new QListViewItem(keyTable, prevMenuLVI);
+			currMenuLVI = new QTreeWidgetItem(keyTable, prevMenuLVI);
 		Q_CHECK_PTR(currMenuLVI);
 		lviToMenuMap.append(currMenuLVI);
 		currMenuLVI->setText(0, itmenu->first);
-		currMenuLVI->setOpen(true);
+		currMenuLVI->setExpanded(true);
 		prevMenuLVI=currMenuLVI;
 		first=true;
 		currLVI=0;
@@ -386,11 +414,11 @@ void TabKeyboardShortcutsWidget::insertActions()
 		{
 			if (first)
 			{
-				currLVI=new QListViewItem(currMenuLVI);
+				currLVI=new QTreeWidgetItem(currMenuLVI);
 				first=false;
 			}
 			else
-				currLVI=new QListViewItem(currMenuLVI, prevLVI);
+				currLVI=new QTreeWidgetItem(currMenuLVI, prevLVI);
 			Q_CHECK_PTR(currLVI);
 			lviToActionMap.insert(currLVI, *it);
 			currLVI->setText(0, keyMap[*it].cleanMenuText);
@@ -398,54 +426,31 @@ void TabKeyboardShortcutsWidget::insertActions()
 			prevLVI=currLVI;
 		}
 	}
+	keyTable->resizeColumnToContents(0);
 }
 
 void TabKeyboardShortcutsWidget::applySearch( const QString & newss )
 {
 	//Must run this as if newss is not empty and we go to the next for loop, the set visible doesnt work
-	for (QPtrList<QListViewItem>::iterator it=lviToMenuMap.begin(); it!=lviToMenuMap.end(); ++it)
-		(*it)->setVisible(true);
+	for (QList<QTreeWidgetItem*>::iterator it=lviToMenuMap.begin(); it!=lviToMenuMap.end(); ++it)
+		(*it)->setHidden(false);
 	if (newss.isEmpty())
 	{			
-		for (QMap<QListViewItem*, QString>::iterator it=lviToActionMap.begin(); it!=lviToActionMap.end(); ++it)
-			it.key()->setVisible(true);
+		for (QMap<QTreeWidgetItem*, QString>::iterator it=lviToActionMap.begin(); it!=lviToActionMap.end(); ++it)
+			it.key()->setHidden(false);
 		return;
 	}
 	//Seem to need to do this.. isOpen doesnt seem to do what it says
-	for (QMap<QListViewItem*, QString>::iterator it=lviToActionMap.begin(); it!=lviToActionMap.end(); ++it)
+	for (QMap<QTreeWidgetItem*, QString>::iterator it=lviToActionMap.begin(); it!=lviToActionMap.end(); ++it)
 	{
-		if (it.key()->text(0).contains(newss, false))
-			it.key()->setVisible(true);
+		if (it.key()->text(0).contains(newss, Qt::CaseInsensitive))
+			it.key()->setHidden(false);
 		else
-			it.key()->setVisible(false);
-	}
-	for (QPtrList<QListViewItem>::iterator it=lviToMenuMap.begin(); it!=lviToMenuMap.end(); ++it)
-	{
-		bool toBeVisible=false;
-		QListViewItem* fc=(*it)->firstChild();
-		if (fc!=0)
-		{
-			if (fc->isVisible())
-				toBeVisible=true;
-			else
-			{
-				QListViewItem* sibling=fc->nextSibling();
-				while (sibling!=0)
-				{
-					if (sibling->isVisible())
-					{
-						toBeVisible=true;
-						break;
-					}
-					sibling=sibling->nextSibling();
-				}
-			}
-		}
-		(*it)->setVisible(toBeVisible);
+			it.key()->setHidden(true);
 	}
 }
 
-void TabKeyboardShortcutsWidget::dispKey(QListViewItem* qlvi)
+void TabKeyboardShortcutsWidget::dispKey(QTreeWidgetItem* qlvi, QTreeWidgetItem*)
 {
 	if (qlvi!=0 && lviToActionMap.contains(qlvi))
 	{
@@ -479,39 +484,42 @@ bool TabKeyboardShortcutsWidget::event( QEvent* ev )
 
 void TabKeyboardShortcutsWidget::keyPressEvent(QKeyEvent *k)
 {
-	if (setKeyButton->isOn())
+	if (setKeyButton->isChecked())
 	{
 		QStringList tl;
+		Part4 = "";
 		if (!keyDisplay->text().isEmpty())
 		{
-			tl = tl.split("+", keyDisplay->text());
-			Part4 = tl[tl.count()-1];
-			if (Part4 == tr("Alt") || Part4 == tr("Ctrl") || Part4 == tr("Shift") || Part4 == tr("Meta"))
-				Part4 = "";
-		}
-		else
-			Part4 = "";
+			tl = keyDisplay->text().split("+", QString::SkipEmptyParts);
+			if (tl.count() > 0)
+			{
+				Part4 = tl[tl.count()-1];
+				if (Part4 == tr("Alt") || Part4 == tr("Ctrl") || Part4 == tr("Shift") || Part4 == tr("Meta"))
+					Part4 = "";
+			}
+		}		
 		switch (k->key())
 		{
-			case Key_Meta:
+			case Qt::Key_Meta:
 				Part0 = tr("Meta+");
 				keyCode |= Qt::META;
 				break;
-			case Key_Shift:
+			case Qt::Key_Shift:
 				Part3 = tr("Shift+");
 				keyCode |= Qt::SHIFT;
 				break;
-			case Key_Alt:
+			case Qt::Key_Alt:
 				Part2 = tr("Alt+");
 				keyCode |= Qt::ALT;
 				break;
-			case Key_Control:
+			case Qt::Key_Control:
 				Part1 = tr("Ctrl+");
 				keyCode |= Qt::CTRL;
 				break;
 			default:
 				keyCode |= k->key();
 				keyDisplay->setText(getKeyText(keyCode));
+				releaseKeyboard();
 				if (checkKey(keyCode))
 				{
 					QMessageBox::information(this, CommonStrings::trWarning,
@@ -527,28 +535,29 @@ void TabKeyboardShortcutsWidget::keyPressEvent(QKeyEvent *k)
 					keyMap[lviToActionMap[selectedLVI]].keySequence=newKeySequence;
 					userDef->setChecked(true);
 				}
-				setKeyButton->setOn(false);
-				releaseKeyboard();
+				setKeyButton->setChecked(false);
 		}
 	}
-	if (setKeyButton->isOn())
+	if (setKeyButton->isChecked())
 		keyDisplay->setText(Part0+Part1+Part2+Part3+Part4);
 }
 
 void TabKeyboardShortcutsWidget::keyReleaseEvent(QKeyEvent *k)
 {
-	if (setKeyButton->isOn())
+	if (setKeyButton->isChecked())
 	{
+		Part4 = "";
 		if (!keyDisplay->text().isEmpty())
 		{
 			QStringList tl;
-			tl = tl.split("+", keyDisplay->text());
-			Part4 = tl[tl.count()-1];
-			if (Part4 == tr("Alt") || Part4 == tr("Ctrl") || Part4 == tr("Shift") || Part4 == tr("Meta"))
-				Part4 = "";
+			tl = keyDisplay->text().split("+", QString::SkipEmptyParts);
+			if (tl.count() > 0)
+			{
+				Part4 = tl[tl.count()-1];
+				if (Part4 == tr("Alt") || Part4 == tr("Ctrl") || Part4 == tr("Shift") || Part4 == tr("Meta"))
+					Part4 = "";
+			}
 		}
-		else
-			Part4 = "";
 		if (k->key() == Qt::Key_Meta)
 		{
 			Part0 = "";
@@ -562,7 +571,7 @@ void TabKeyboardShortcutsWidget::keyReleaseEvent(QKeyEvent *k)
 		if (k->key() == Qt::Key_Alt)
 		{
 			Part2 = "";
-			keyCode &= ~ALT;
+			keyCode &= ~Qt::ALT;
 		}
 		if (k->key() == Qt::Key_Control)
 		{
@@ -579,7 +588,7 @@ bool TabKeyboardShortcutsWidget::checkKey(int code)
 	QKeySequence key = QKeySequence(code);
 	for (QMap<QString,Keys>::Iterator it=keyMap.begin(); it!=keyMap.end(); ++it)
 	{
-		if (key.matches(it.data().keySequence) != Qt::NoMatch)
+		if (key.matches(it.value().keySequence) != QKeySequence::NoMatch)
 		{
 			ret = true;
 			break;
